"""
Allows the parsing of python-style regexes to FSMs.
Main access point is `parse_pattern(str) -> Pattern`.
Most other classes are internal and should not be used.
"""

from abc import abstractmethod, ABC
from dataclasses import dataclass
from enum import Flag, auto
from textwrap import indent
from typing import Iterable, FrozenSet, Optional, Tuple, Union

from interegular.fsm import FSM, anything_else, epsilon, Alphabet
from interegular.utils.simple_parser import SimpleParser, nomatch, NoMatch

__all__ = ['parse_pattern', 'Pattern', 'Unsupported', 'InvalidSyntax', 'REFlags']


class Unsupported(Exception):
    pass


class InvalidSyntax(Exception):
    pass


class REFlags(Flag):
    CASE_INSENSITIVE = I = auto()
    MULTILINE = M = auto()
    SINGLE_LINE = S = auto()


_flags = {
    'i': REFlags.I,
    'm': REFlags.M,
    's': REFlags.S,
}


def _get_flags(plus: str) -> REFlags:
    res = REFlags(0)
    for c in plus:
        try:
            res |= _flags[c]
        except KeyError:
            raise Unsupported(f"Flag {c} is not implemented")
    return res


def _combine_flags(base: REFlags, added: REFlags, removed: REFlags):
    base |= added
    base &= ~removed
    # TODO: Check for incorrect combinations (aLu)
    return base


@dataclass(frozen=True)
class _BasePattern(ABC):
    __slots__ = '_alphabet_cache', '_prefix_cache', '_lengths_cache'

    @abstractmethod
    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=None) -> FSM:
        raise NotImplementedError

    @abstractmethod
    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        raise NotImplementedError

    def get_alphabet(self, flags: REFlags) -> Alphabet:
        if not hasattr(self, '_alphabet_cache'):
            super(_BasePattern, self).__setattr__('_alphabet_cache', {})
        if flags not in self._alphabet_cache:
            self._alphabet_cache[flags] = self._get_alphabet(flags)
        return self._alphabet_cache[flags]

    @abstractmethod
    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        raise NotImplementedError

    @property
    def prefix_postfix(self) -> Tuple[int, Optional[int]]:
        """Returns the number of dots that have to be pre-/postfixed to support look(aheads|backs)"""
        if not hasattr(self, '_prefix_cache'):
            super(_BasePattern, self).__setattr__('_prefix_cache', self._get_prefix_postfix())
        return self._prefix_cache

    @abstractmethod
    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        raise NotImplementedError

    @property
    def lengths(self) -> Tuple[int, Optional[int]]:
        """Returns the minimum and maximum length that this pattern can match
         (maximum can be None bei infinite length)"""
        if not hasattr(self, '_lengths_cache'):
            super(_BasePattern, self).__setattr__('_lengths_cache', self._get_lengths())
        return self._lengths_cache

    @abstractmethod
    def simplify(self) -> '_BasePattern':
        raise NotImplementedError


class _Repeatable(_BasePattern, ABC):
    pass


@dataclass(frozen=True)
class _CharGroup(_Repeatable):
    """Represents the smallest possible pattern that can be matched: A single char.
    Direct port from the lego module"""
    chars: FrozenSet[str]
    negated: bool
    __slots__ = 'chars', 'negated'

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        if flags & REFlags.CASE_INSENSITIVE:
            relevant = {*map(str.lower, self.chars), *map(str.upper, self.chars)}
        else:
            relevant = self.chars
        return Alphabet.from_groups(relevant, {anything_else})

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        return 0, 0

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        return 1, 1

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        if prefix_postfix is None:
            prefix_postfix = self.prefix_postfix
        if prefix_postfix != (0, 0):
            raise ValueError("Can not have prefix/postfix on CharGroup-level")
        insensitive = flags & REFlags.CASE_INSENSITIVE
        flags &= ~REFlags.CASE_INSENSITIVE
        flags &= ~REFlags.SINGLE_LINE
        if flags:
            raise Unsupported(flags)
        if insensitive:
            chars = frozenset({*(c.lower() for c in self.chars), *(c.upper() for c in self.chars)})
        else:
            chars = self.chars

        # State: 0 is initial, 1 is final

        # If negated, make a singular FSM accepting any other characters
        if self.negated:
            mapping = {
                0: {alphabet[symbol]: 1 for symbol in set(alphabet) - chars},
            }

        # If normal, make a singular FSM accepting only these characters
        else:
            mapping = {
                0: {alphabet[symbol]: 1 for symbol in chars},
            }

        return FSM(
            alphabet=alphabet,
            states={0, 1},
            initial=0,
            finals={1},
            map=mapping,
        )

    def simplify(self) -> '_CharGroup':
        return self


def _combine_char_groups(*groups: _CharGroup, negate):
    pos = set().union(*(g.chars for g in groups if not g.negated))
    neg = set().union(*(g.chars for g in groups if g.negated))
    if neg:
        return _CharGroup(frozenset(neg - pos), not negate)
    else:
        return _CharGroup(frozenset(pos - neg), negate)


@dataclass(frozen=True)
class __DotCls(_Repeatable):

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        if flags is None or not flags & REFlags.SINGLE_LINE:
            symbols = set(alphabet) - {'\n'}
        else:
            symbols = alphabet
        return FSM(
            alphabet=alphabet,
            states={0, 1},
            initial=0,
            finals={1},
            map={0: {alphabet[sym]: 1 for sym in symbols}},
        )

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        if flags & REFlags.SINGLE_LINE:
            return Alphabet.from_groups({anything_else})
        else:
            return Alphabet.from_groups({anything_else}, {'\n'})

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        return 0, 0

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        return 1, 1

    def simplify(self) -> '__DotCls':
        return self


@dataclass(frozen=True)
class __EmptyCls(_BasePattern):

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        return epsilon(alphabet)

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        return Alphabet.from_groups({anything_else})

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        return 0, 0

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        return 0, 0

    def simplify(self) -> '__EmptyCls':
        return self


_DOT = __DotCls()
_EMPTY = __EmptyCls()
_NONE = _CharGroup(frozenset(""), False)
_ALL = _CharGroup(frozenset(""), True)
_CHAR_GROUPS = {
    'w': _CharGroup(frozenset("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"), False),
    'W': _CharGroup(frozenset("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"), True),
    'd': _CharGroup(frozenset("0123456789"), False),
    'D': _CharGroup(frozenset("0123456789"), True),
    's': _CharGroup(frozenset(" \t\n\r\f\v"), False),
    'S': _CharGroup(frozenset(" \t\n\r\f\v"), True),

    'a': _CharGroup(frozenset("\a"), False),
    'b': _CharGroup(frozenset("\b"), False),
    'f': _CharGroup(frozenset("\f"), False),
    'n': _CharGroup(frozenset("\n"), False),
    'r': _CharGroup(frozenset("\r"), False),
    't': _CharGroup(frozenset("\t"), False),
    'v': _CharGroup(frozenset("\v"), False),
}


@dataclass(frozen=True)
class _Repeated(_BasePattern):
    """Represents a repeated pattern. `base` can be matched from `min` to `max` times.
    `max` may be None to signal infinite"""
    base: _Repeatable
    min: int
    max: Optional[int]

    def __str__(self):
        return f"Repeated[{self.min}:{self.max if self.max is not None else ''}]:\n" \
            f"{indent(str(self.base), '    ')}"

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        return self.base.get_alphabet(flags)

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        return self.base.prefix_postfix

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        l, h = self.base.lengths
        return l * self.min, (h * self.max if None not in (h, self.max) else None)

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        if prefix_postfix is None:
            prefix_postfix = self.prefix_postfix
        if prefix_postfix != (0, 0):
            raise ValueError("Can not have prefix/postfix on CharGroup-level")

        unit = self.base.to_fsm(alphabet, (0, 0), flags=flags)
        mandatory = unit * self.min
        if self.max is None:
            optional = unit.star()
        else:
            optional = unit.copy()
            optional.__dict__['finals'] |= {optional.initial}
            optional *= (self.max - self.min)
        return mandatory + optional

    def simplify(self) -> '_Repeated':
        return self.__class__(self.base.simplify(), self.min, self.max)


_ALL_STAR = _Repeated(_ALL, 0, None)


@dataclass(frozen=True)
class _NonCapturing:
    """Represents a lookahead/lookback. Matches `inner` without 'consuming' anything. Can be negated.
    Only valid inside a `_Concatenation`"""
    inner: _BasePattern
    backwards: bool
    negate: bool
    __slots__ = 'inner', 'backwards', 'negate'

    def get_alphabet(self, flags: REFlags) -> Alphabet:
        return self.inner.get_alphabet(flags)

    def simplify(self) -> '_NonCapturing':
        return self.__class__(self.inner.simplify(), self.backwards, self.negate)


@dataclass(frozen=True)
class _Concatenation(_BasePattern):
    """Represents multiple Patterns that have to be match in a row. Can contain `_NonCapturing`"""
    parts: Tuple[Union[_BasePattern, _NonCapturing], ...]
    __slots__ = 'parts',

    def __str__(self):
        return "Concatenation:\n" + "\n".join(indent(str(p), '  ') for p in self.parts)

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        return Alphabet.union(*(p.get_alphabet(flags) for p in self.parts))[0]

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        pre = 0  # What is the longest a lookback could stick out over the beginning?
        off = 0  # How many chars have been consumed, e.g what is the minimum length?
        for p in self.parts:
            if not isinstance(p, _NonCapturing):
                off += p.lengths[0]
            elif p.backwards:
                a, b = p.inner.lengths
                if a != b:
                    raise InvalidSyntax(f"lookbacks have to have fixed length {(a, b)}")
                req = a - off
                if req > pre:
                    pre = req
        post = 0
        off = 0
        for p in reversed(self.parts):
            if not isinstance(p, _NonCapturing):
                off += p.lengths[0]
            elif not p.backwards:
                a, b = p.inner.lengths
                if b is None:
                    req = a - off  # TODO: is this correct?
                else:
                    req = b - off
                if req > post:
                    post = req
        return pre, post

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        low, high = 0, 0
        for p in self.parts:
            if not isinstance(p, _NonCapturing):
                pl, ph = p.lengths
                low += pl
                high = high + ph if None not in (high, ph) else None
        return low, high

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        if prefix_postfix is None:
            prefix_postfix = self.prefix_postfix
        if prefix_postfix[0] < self.prefix_postfix[0] or prefix_postfix[1] < self.prefix_postfix[1]:
            raise Unsupported("Group can not have lookbacks/lookaheads that go beyond the group bounds.")

        all_ = _ALL.to_fsm(alphabet)
        all_star = all_.star()
        fsm_parts = []
        current = [all_.times(prefix_postfix[0])]
        for part in self.parts:
            if isinstance(part, _NonCapturing):
                inner = part.inner.to_fsm(alphabet, (0, 0), flags)
                if part.backwards:
                    raise Unsupported("lookbacks are not implemented")
                else:
                    # try:
                    #     inner.cardinality()
                    # except OverflowError:
                    #     raise NotImplementedError("Can not deal with infinite length lookaheads")
                    fsm_parts.append((None, current))
                    fsm_parts.append((part, inner))
                    current = []
            else:
                current.append(part.to_fsm(alphabet, (0, 0), flags))
        current.append(all_.times(prefix_postfix[1]))
        result = FSM.concatenate(*current)
        for m, f in reversed(fsm_parts):
            if m is None:
                result = FSM.concatenate(*f, result)
            else:
                assert isinstance(m, _NonCapturing) and not m.backwards
                if m.negate:
                    result = result.difference(f + all_star)  # TODO: This does not feel right...
                else:
                    result = result.intersection(f + all_star)
        return result

    def simplify(self) -> '_Concatenation':
        return self.__class__(tuple(p.simplify() for p in self.parts))


@dataclass(frozen=True)
class Pattern(_Repeatable):
    options: Tuple[_BasePattern, ...]
    added_flags: REFlags = REFlags(0)
    removed_flags: REFlags = REFlags(0)

    def __str__(self):
        return "Pattern:\n" + "\n".join(indent(str(o), '  ') for o in self.options)

    def _get_alphabet(self, flags: REFlags) -> Alphabet:
        flags = _combine_flags(flags, self.added_flags, self.removed_flags)
        return Alphabet.union(*(p.get_alphabet(flags) for p in self.options))[0]

    def _get_lengths(self) -> Tuple[int, Optional[int]]:
        low, high = None, 0
        for o in self.options:
            ol, oh = o.lengths
            if low is None or ol < low:
                low = ol
            if oh is None or (high is not None and oh > high):
                high = oh
        return low, high

    def _get_prefix_postfix(self) -> Tuple[int, Optional[int]]:
        pre, post = 0, 0
        for o in self.options:
            opre, opost = o.prefix_postfix
            if opre > pre:
                pre = opre
            if opost is None or (post is not None and opost > post):
                post = opost
        return pre, post

    def to_fsm(self, alphabet=None, prefix_postfix=None, flags=REFlags(0)) -> FSM:
        flags = _combine_flags(flags, self.added_flags, self.removed_flags)
        if alphabet is None:
            alphabet = self.get_alphabet(flags)
        if prefix_postfix is None:
            prefix_postfix = self.prefix_postfix
        return FSM.union(*(o.to_fsm(alphabet, prefix_postfix, flags) for o in self.options))

    def with_flags(self, added: REFlags, removed: REFlags = REFlags(0)) -> 'Pattern':
        return self.__class__(self.options, added, removed)

    def simplify(self) -> 'Pattern':
        if len(self.options) == 1:
            o = self.options[0]
            if isinstance(o, _Concatenation) and len(o.parts) == 1 and isinstance(o.parts[0], Pattern):
                p: Pattern = o.parts[0].simplify()
                f = _combine_flags(_combine_flags(REFlags(0), self.added_flags, self.removed_flags),
                                   p.added_flags, p.removed_flags)
                return p.with_flags(f)
        return self.__class__(tuple(o.simplify() for o in self.options), self.added_flags, self.removed_flags)


class _ParsePattern(SimpleParser[Pattern]):
    SPECIAL_CHARS_STANDARD: FrozenSet[str] = frozenset({
        '+', '?', '*', '.', '$', '^', '\\', '(', ')', '[', '|'
    })
    SPECIAL_CHARS_INNER: FrozenSet[str] = frozenset({
        '\\', ']'
    })
    RESERVED_ESCAPES: FrozenSet[str] = frozenset({
        'u', 'U', 'A', 'Z', 'b', 'B'
    })

    def __init__(self, data: str):
        super(_ParsePattern, self).__init__(data)
        self.flags = None

    def parse(self):
        try:
            return super(_ParsePattern, self).parse()
        except NoMatch:
            raise InvalidSyntax

    def start(self):
        self.flags = None
        p = self.pattern()
        if self.flags is not None:
            p = p.with_flags(self.flags)
        return p

    def pattern(self):
        options = [self.conc()]
        while self.static_b('|'):
            options.append(self.conc())
        return Pattern(tuple(options))

    def conc(self):
        parts = []
        while True:
            try:
                parts.append(self.obj())
            except nomatch:
                break
        return _Concatenation(tuple(parts))

    def obj(self):
        if self.static_b("("):
            return self.group()
        return self.repetition(self.atom())

    def group(self):
        if self.static_b("?"):
            return self.extension_group()
        else:
            p = self.pattern()
            self.static(")")
            return self.repetition(p)

    def extension_group(self):
        c = self.any()
        if c in 'aiLmsux-':
            self.index -= 1
            added_flags = self.multiple('aiLmsux', 0, None)
            if self.static_b('-'):
                removed_flags = self.multiple('aiLmsux', 1, None)
            else:
                removed_flags = ''
            if self.static_b(':'):
                p = self.pattern()
                p = p.with_flags(_get_flags(added_flags), _get_flags(removed_flags))
                self.static(")")
                return self.repetition(p)
            elif removed_flags != '':
                raise nomatch
            else:
                self.static(')')
                self.flags = _get_flags(added_flags)
                return _EMPTY
        elif c == ':':
            p = self.pattern()
            self.static(")")
            return self.repetition(p)
        elif c == 'P':
            if self.static_b('<'):
                self.multiple('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_', 1, None)
                self.static('>')
                p = self.pattern()
                self.static(")")
                return self.repetition(p)
            elif self.static_b('='):
                raise Unsupported("Group references are not implemented")
        elif c == '#':
            while not self.static_b(')'):
                self.any()
        elif c == '=':
            p = self.pattern()
            self.static(")")
            return _NonCapturing(p, False, False)
        elif c == '!':
            p = self.pattern()
            self.static(")")
            return _NonCapturing(p, False, True)
        elif c == '<':
            c = self.any()
            if c == '=':
                p = self.pattern()
                self.static(")")
                return _NonCapturing(p, True, False)
            elif c == '!':
                p = self.pattern()
                self.static(")")
                return _NonCapturing(p, True, True)
        elif c == '(':
            raise Unsupported("Conditional matching is not implemented")
        else:
            raise InvalidSyntax(
                f"Unknown group-extension: {c!r} (Context: {self.data[self.index - 3:self.index + 5]!r}")

    def atom(self):
        if self.static_b("["):
            return self.repetition(self.chargroup())
        elif self.static_b("\\"):
            return self.repetition(self.escaped())
        elif self.static_b("."):
            return self.repetition(_DOT)
        elif self.static_b("$"):
            raise Unsupported("'$'")
        elif self.static_b("^"):
            raise Unsupported("'^'")
        else:
            c = self.any_but(*self.SPECIAL_CHARS_STANDARD)
            return self.repetition(_CharGroup(frozenset({c}), False))

    def repetition(self, base: _Repeatable):
        if self.static_b("*"):
            if self.static_b("?"):
                pass
            return _Repeated(base, 0, None)
        elif self.static_b("+"):
            if self.static_b("?"):
                pass
            return _Repeated(base, 1, None)
        elif self.static_b("?"):
            if self.static_b("?"):
                pass
            return _Repeated(base, 0, 1)
        elif self.static_b("{"):
            try:
                n = self.number()
            except nomatch:
                n = 0
            if self.static_b(','):
                try:
                    m = self.number()
                except nomatch:
                    m = None
            else:
                m = n
            self.static("}")
            if self.static_b('?'):
                pass
            return _Repeated(base, n, m)
        else:
            return base

    def number(self) -> int:
        return int(self.multiple("0123456789", 1, None))

    def escaped(self, inner=False):
        if self.static_b("x"):
            n = self.multiple("0123456789abcdefABCDEF", 2, 2)
            c = chr(int(n, 16))
            return _CharGroup(frozenset({c}), False)
        if self.static_b("0"):
            n = self.multiple("01234567", 1, 2)
            c = chr(int(n, 8))
            return _CharGroup(frozenset({c}), False)
        if self.anyof_b('N', 'p', 'P', 'u', 'U'):
            raise Unsupported('regex module unicode properties are not supported.')
        if not inner:
            try:
                n = self.multiple("01234567", 3, 3)
            except nomatch:
                pass
            else:
                c = chr(int(n, 8))
                return _CharGroup(frozenset({c}), False)
            try:
                self.multiple("0123456789", 1, 2)
            except nomatch:
                pass
            else:
                raise Unsupported("Group references are not implemented")
        else:
            try:
                n = self.multiple("01234567", 1, 3)
            except nomatch:
                pass
            else:
                c = chr(int(n, 8))
                return _CharGroup(frozenset({c}), False)
        if not inner:
            try:
                c = self.anyof(*self.RESERVED_ESCAPES)
            except nomatch:
                pass
            else:
                raise Unsupported(f"Escape \\{c} is not implemented")
        try:
            c = self.anyof(*_CHAR_GROUPS)
        except nomatch:
            pass
        else:
            return _CHAR_GROUPS[c]
        c = self.any_but("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
        if c.isalpha():
            raise nomatch
        return _CharGroup(frozenset(c), False)

    def chargroup(self):
        if self.static_b("^"):
            negate = True
        else:
            negate = False
        groups = []
        while True:
            try:
                groups.append(self.chargroup_inner())
            except nomatch:
                break
        self.static("]")
        if len(groups) == 1:
            f = tuple(groups)[0]
            return _CharGroup(f.chars, negate ^ f.negated)
        elif len(groups) == 0:
            return _CharGroup(frozenset({}), negate)
        else:
            return _combine_char_groups(*groups, negate=negate)

    def chargroup_inner(self) -> _CharGroup:
        start = self.index
        if self.static_b('\\'):
            base = self.escaped(True)
        else:
            base = _CharGroup(frozenset(self.any_but(*self.SPECIAL_CHARS_INNER)), False)
        if self.static_b('-'):
            if self.static_b('\\'):
                end = self.escaped(True)
            elif self.peek_static(']'):
                return _combine_char_groups(base, _CharGroup(frozenset('-'), False), negate=False)
            else:
                end = _CharGroup(frozenset(self.any_but(*self.SPECIAL_CHARS_INNER)), False)
            if len(base.chars) != 1 or len(end.chars) != 1:
                raise InvalidSyntax(f"Invalid Character-range: {self.data[start:self.index]}")
            low, high = ord(*base.chars), ord(*end.chars)
            if low > high:
                raise InvalidSyntax(f"Invalid Character-range: {self.data[start:self.index]}")
            return _CharGroup(frozenset((chr(i) for i in range(low, high + 1))), False)
        return base


def parse_pattern(pattern: str) -> Pattern:
    p = _ParsePattern(pattern)
    out = p.parse()
    out = out.simplify()
    return out



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Transform(VaultApiBase):
    def create_or_update_role(self, name, transformations, mount_point: str = "transform"): ...
    def read_role(self, name, mount_point: str = "transform"): ...
    def list_roles(self, mount_point: str = "transform"): ...
    def delete_role(self, name, mount_point: str = "transform"): ...
    def create_or_update_transformation(
        self,
        name,
        transform_type,
        template,
        tweak_source: str = "supplied",
        masking_character: str = "*",
        allowed_roles=None,
        mount_point: str = "transform",
    ): ...
    def create_or_update_fpe_transformation(
        self, name, template, tweak_source: str = "supplied", allowed_roles=None, mount_point: str = "transform"
    ): ...
    def create_or_update_masking_transformation(
        self, name, template, masking_character: str = "*", allowed_roles=None, mount_point: str = "transform"
    ): ...
    def create_or_update_tokenization_transformation(
        self,
        name,
        max_ttl: int = 0,
        mapping_mode: str = "default",
        allowed_roles=None,
        stores=None,
        mount_point: str = "transform",
    ): ...
    def read_transformation(self, name, mount_point: str = "transform"): ...
    def list_transformations(self, mount_point: str = "transform"): ...
    def delete_transformation(self, name, mount_point: str = "transform"): ...
    def create_or_update_template(self, name, template_type, pattern, alphabet, mount_point: str = "transform"): ...
    def read_template(self, name, mount_point: str = "transform"): ...
    def list_templates(self, mount_point: str = "transform"): ...
    def delete_template(self, name, mount_point: str = "transform"): ...
    def create_or_update_alphabet(self, name, alphabet, mount_point: str = "transform"): ...
    def read_alphabet(self, name, mount_point: str = "transform"): ...
    def list_alphabets(self, mount_point: str = "transform"): ...
    def delete_alphabet(self, name, mount_point: str = "transform"): ...
    def create_or_update_tokenization_store(
        self,
        name,
        driver,
        connection_string,
        username=None,
        password=None,
        type: str = "sql",
        supported_transformations=None,
        schema: str = "public",
        max_open_connections: int = 4,
        max_idle_connections: int = 4,
        max_connection_lifetime: int = 0,
        mount_point: str = "transform",
    ): ...
    def encode(
        self, role_name, value=None, transformation=None, tweak=None, batch_input=None, mount_point: str = "transform"
    ): ...
    def decode(
        self, role_name, value=None, transformation=None, tweak=None, batch_input=None, mount_point: str = "transform"
    ): ...
    def validate_token(self, role_name, value, transformation, batch_input=None, mount_point: str = "transform"): ...
    def check_tokenization(self, role_name, value, transformation, batch_input=None, mount_point: str = "transform"): ...
    def retrieve_token_metadata(self, role_name, value, transformation, batch_input=None, mount_point: str = "transform"): ...
    def snapshot_tokenization_state(self, name, limit: int = 1000, continuation: str = "", mount_point: str = "transform"): ...
    def restore_tokenization_state(self, name, values, mount_point: str = "transform"): ...
    def export_decoded_tokenization_state(
        self, name, limit: int = 1000, continuation: str = "", mount_point: str = "transform"
    ): ...
    def rotate_tokenization_key(self, transform_name, mount_point: str = "transform"): ...
    def update_tokenization_key_config(self, transform_name, min_decryption_version, mount_point: str = "transform"): ...
    def list_tokenization_key_configuration(self, mount_point: str = "transform"): ...
    def read_tokenization_key_configuration(self, transform_name, mount_point: str = "transform"): ...
    def trim_tokenization_key_version(self, transform_name, min_available_version, mount_point: str = "transform"): ...



from collections.abc import Collection

import numpy as np
from networkx._typing import Array2D
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["modularity_matrix", "directed_modularity_matrix"]

@_dispatchable
def modularity_matrix(
    G: Graph[_Node], nodelist: Collection[_Node] | None = None, weight: str | None = None
) -> Array2D[np.float64]: ...
@_dispatchable
def directed_modularity_matrix(
    G: DiGraph[_Node], nodelist: Collection[_Node] | None = None, weight: str | None = None
) -> Array2D[np.float64]: ...



from _typeshed import Incomplete
from collections.abc import Sequence
from typing import Final

__version__: Final[str]
CANNOT_START_LINE: Final[Sequence[str]]
ALL_CANNOT_START: Final[str]
CANNOT_END_LINE: Final[Sequence[str]]
ALL_CANNOT_END: Final[str]

def is_multi_byte(ch): ...
def getCharWidths(word, fontName, fontSize): ...
def wordSplit(word, maxWidths, fontName, fontSize, encoding: str = "utf8"): ...
def dumbSplit(word, widths, maxWidths): ...
def kinsokuShoriSplit(word, widths, availWidth) -> None: ...

rx: Incomplete

def cjkwrap(text, width, encoding: str = "utf8"): ...



def nullTransform(): ...
def translate(dx: float, dy: float = 0): ...
def scale(sx: float, sy: float = 1): ...
def rotate(angle: float, cx: float = 0, cy: float = 0): ...
def skewX(angle: float): ...
def skewY(angle: float): ...
def mmult(A, B): ...
def inverse(A): ...
def zTransformPoint(A, v): ...
def transformPoint(A, v): ...
def transformPoints(matrix, V): ...
def zTransformPoints(matrix, V): ...

__all__ = (
    "nullTransform",
    "translate",
    "scale",
    "rotate",
    "skewX",
    "skewY",
    "mmult",
    "inverse",
    "zTransformPoint",
    "transformPoint",
    "transformPoints",
    "zTransformPoints",
)



from _typeshed import Incomplete

from reportlab.graphics.charts.areas import PlotArea
from reportlab.lib.attrmap import *

class Ean13BarcodeWidget(PlotArea):
    codeName: str
    barHeight: Incomplete
    barWidth: Incomplete
    humanReadable: int
    quiet: int
    rquiet: Incomplete
    lquiet: Incomplete
    fontSize: int
    fontName: str
    textColor: Incomplete
    barFillColor: Incomplete
    barStrokeColor: Incomplete
    barStrokeWidth: int
    x: int
    y: int
    value: Incomplete
    def __init__(self, value: str = "123456789012", **kw) -> None: ...
    @property
    def width(self): ...  # type: ignore[override]
    def wrap(self, aW, aH): ...
    def draw(self): ...

class Ean8BarcodeWidget(Ean13BarcodeWidget):
    codeName: str

class UPCA(Ean13BarcodeWidget):
    codeName: str

class Ean5BarcodeWidget(Ean13BarcodeWidget):
    codeName: str
    def draw(self): ...

class ISBNBarcodeWidget(Ean13BarcodeWidget):
    codeName: str
    def draw(self): ...

__all__ = ("Ean13BarcodeWidget", "Ean8BarcodeWidget", "UPCA", "Ean5BarcodeWidget", "ISBNBarcodeWidget")



from authlib.oidc.core import UserInfo

def is_openid_scope(scope: str | None) -> bool: ...
def validate_request_prompt(grant, redirect_uri, redirect_fragment: bool = False): ...
def validate_nonce(request, exists_nonce, required: bool = False): ...
def generate_id_token(
    token: dict[str, str | int],
    user_info: UserInfo,
    key: str,
    iss: str,
    aud: list[str],
    alg: str = "RS256",
    exp: int = 3600,
    nonce: str | None = None,
    auth_time: int | None = None,
    acr: str | None = None,
    amr: list[str] | None = None,
    code: str | None = None,
    kid: str | None = None,
) -> str: ...
def create_response_mode_response(redirect_uri, params, response_mode): ...



from typing import Any

signals_available: bool

class Namespace:
    def signal(self, name: str, doc: str | None = None) -> _FakeSignal: ...

class _FakeSignal:
    name: str
    __doc__: Any
    def __init__(self, name: str, doc: str | None = None) -> None: ...
    send: Any
    connect: Any
    disconnect: Any
    has_receivers_for: Any
    receivers_for: Any
    temporarily_connected_to: Any
    connected_to: Any

scope_changed: _FakeSignal



from _typeshed import SupportsKeysAndGetItem, Unused
from typing import Final, Literal
from typing_extensions import Self, TypeAlias

from .jks import TrustedCertEntry
from .util import AbstractKeystore, AbstractKeystoreEntry

_BksType: TypeAlias = Literal["bks", "uber"]
_CertType: TypeAlias = Literal["X.509"]
_EntryFormat: TypeAlias = Literal["PKCS8", "PKCS#8", "X.509", "X509", "RAW"]
_BksVersion: TypeAlias = Literal[1, 2]

ENTRY_TYPE_CERTIFICATE: Final = 1
ENTRY_TYPE_KEY: Final = 2
ENTRY_TYPE_SECRET: Final = 3
ENTRY_TYPE_SEALED: Final = 4

KEY_TYPE_PRIVATE: Final = 0
KEY_TYPE_PUBLIC: Final = 1
KEY_TYPE_SECRET: Final = 2
_KeyType: TypeAlias = Literal[0, 1, 2]

class AbstractBksEntry(AbstractKeystoreEntry):
    store_type: _BksType | None
    cert_chain: list[tuple[_CertType, bytes]]
    def __init__(
        self,
        *,
        cert_chain: list[tuple[_CertType, bytes]] = ...,
        encrypted: bytes | None = None,
        store_type: _BksType | None = None,
        alias: str,
        timestamp: int,
        **kwargs: Unused,
    ) -> None: ...

class BksTrustedCertEntry(TrustedCertEntry):
    store_type: _BksType | None  # type: ignore[assignment]

class BksKeyEntry(AbstractBksEntry):
    type: _KeyType
    format: _EntryFormat
    algorithm: str
    encoded: bytes
    # type == KEY_TYPE_PRIVATE
    pkey_pkcs8: bytes
    pkey: bytes
    algorithm_oid: tuple[int, ...]
    # type == KEY_TYPE_PUBLIC
    public_key_info: bytes
    public_key: bytes
    # type == KEY_TYPE_SECRET
    key: bytes
    key_size: int
    def __init__(
        self,
        type: _KeyType,
        format: _EntryFormat,
        algorithm: str,
        encoded: bytes,
        *,
        cert_chain: list[tuple[_CertType, bytes]] = ...,
        encrypted: bytes | None = None,
        store_type: _BksType | None = None,
        alias: str,
        timestamp: int,
        **kwargs: Unused,
    ) -> None: ...
    @classmethod
    def type2str(cls, t: _KeyType) -> Literal["PRIVATE", "PUBLIC", "SECRET"]: ...
    def is_decrypted(self) -> Literal[True]: ...

class BksSecretKeyEntry(AbstractBksEntry):
    key: bytes
    def is_decrypted(self) -> Literal[True]: ...

class BksSealedKeyEntry(AbstractBksEntry):
    # Properties provided by __getattr__
    nested: BksKeyEntry | None
    # __getattr__ proxies all attributes of nested BksKeyEntry after decrypting
    type: _KeyType
    format: _EntryFormat
    algorithm: str
    encoded: bytes
    # if type == KEY_TYPE_PRIVATE
    pkey_pkcs8: bytes
    pkey: bytes
    algorithm_oid: tuple[int, ...]
    # if type == KEY_TYPE_PUBLIC
    public_key_info: bytes
    public_key: bytes
    # if type == KEY_TYPE_SECRET
    key: bytes
    key_size: int

class BksKeyStore(AbstractKeystore):
    store_type: Literal["bks"]
    entries: dict[str, BksTrustedCertEntry | BksKeyEntry | BksSealedKeyEntry | BksSecretKeyEntry]  # type: ignore[assignment]
    version: _BksVersion
    def __init__(
        self,
        store_type: Literal["bks"],
        entries: SupportsKeysAndGetItem[str, BksTrustedCertEntry | BksKeyEntry | BksSealedKeyEntry | BksSecretKeyEntry],
        version: _BksVersion = 2,
    ) -> None: ...
    @property
    def certs(self) -> dict[str, BksTrustedCertEntry]: ...
    @property
    def plain_keys(self) -> dict[str, BksKeyEntry]: ...
    @property
    def sealed_keys(self) -> dict[str, BksSealedKeyEntry]: ...
    @property
    def secret_keys(self) -> dict[str, BksSecretKeyEntry]: ...
    @classmethod
    def loads(cls, data: bytes, store_password: str, try_decrypt_keys: bool = True) -> Self: ...

class UberKeyStore(BksKeyStore):
    store_type: Literal["uber"]  # type: ignore[assignment]
    version: Literal[1]
    def __init__(
        self,
        store_type: Literal["uber"],
        entries: SupportsKeysAndGetItem[str, BksTrustedCertEntry | BksKeyEntry | BksSealedKeyEntry | BksSecretKeyEntry],
        version: Literal[1] = 1,
    ) -> None: ...



import sys
from typing import Literal

if sys.version_info >= (3, 14):
    import enum
    import types
    from _typeshed import AnnotateFunc, AnnotationForm, EvaluateFunc, SupportsItems
    from collections.abc import Mapping
    from typing import Any, ParamSpec, TypeVar, TypeVarTuple, final, overload
    from warnings import deprecated

    __all__ = [
        "Format",
        "ForwardRef",
        "call_annotate_function",
        "call_evaluate_function",
        "get_annotate_from_class_namespace",
        "get_annotations",
        "annotations_to_string",
        "type_repr",
    ]

    class Format(enum.IntEnum):
        VALUE = 1
        VALUE_WITH_FAKE_GLOBALS = 2
        FORWARDREF = 3
        STRING = 4

    @final
    class ForwardRef:
        __slots__ = (
            "__forward_is_argument__",
            "__forward_is_class__",
            "__forward_module__",
            "__weakref__",
            "__arg__",
            "__globals__",
            "__extra_names__",
            "__code__",
            "__ast_node__",
            "__cell__",
            "__owner__",
            "__stringifier_dict__",
        )
        __forward_is_argument__: bool
        __forward_is_class__: bool
        __forward_module__: str | None
        def __init__(
            self, arg: str, *, module: str | None = None, owner: object = None, is_argument: bool = True, is_class: bool = False
        ) -> None: ...
        @overload
        def evaluate(
            self,
            *,
            globals: dict[str, Any] | None = None,
            locals: Mapping[str, Any] | None = None,
            type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...] | None = None,
            owner: object = None,
            format: Literal[Format.STRING],
        ) -> str: ...
        @overload
        def evaluate(
            self,
            *,
            globals: dict[str, Any] | None = None,
            locals: Mapping[str, Any] | None = None,
            type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...] | None = None,
            owner: object = None,
            format: Literal[Format.FORWARDREF],
        ) -> AnnotationForm | ForwardRef: ...
        @overload
        def evaluate(
            self,
            *,
            globals: dict[str, Any] | None = None,
            locals: Mapping[str, Any] | None = None,
            type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...] | None = None,
            owner: object = None,
            format: Format = Format.VALUE,  # noqa: Y011
        ) -> AnnotationForm: ...
        @deprecated("Use `ForwardRef.evaluate()` or `typing.evaluate_forward_ref()` instead.")
        def _evaluate(
            self,
            globalns: dict[str, Any] | None,
            localns: Mapping[str, Any] | None,
            type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...] = ...,
            *,
            recursive_guard: frozenset[str],
        ) -> AnnotationForm: ...
        @property
        def __forward_arg__(self) -> str: ...
        @property
        def __forward_code__(self) -> types.CodeType: ...
        def __eq__(self, other: object) -> bool: ...
        def __hash__(self) -> int: ...
        def __or__(self, other: Any) -> types.UnionType: ...
        def __ror__(self, other: Any) -> types.UnionType: ...

    @overload
    def call_evaluate_function(evaluate: EvaluateFunc, format: Literal[Format.STRING], *, owner: object = None) -> str: ...
    @overload
    def call_evaluate_function(
        evaluate: EvaluateFunc, format: Literal[Format.FORWARDREF], *, owner: object = None
    ) -> AnnotationForm | ForwardRef: ...
    @overload
    def call_evaluate_function(evaluate: EvaluateFunc, format: Format, *, owner: object = None) -> AnnotationForm: ...
    @overload
    def call_annotate_function(
        annotate: AnnotateFunc, format: Literal[Format.STRING], *, owner: object = None
    ) -> dict[str, str]: ...
    @overload
    def call_annotate_function(
        annotate: AnnotateFunc, format: Literal[Format.FORWARDREF], *, owner: object = None
    ) -> dict[str, AnnotationForm | ForwardRef]: ...
    @overload
    def call_annotate_function(annotate: AnnotateFunc, format: Format, *, owner: object = None) -> dict[str, AnnotationForm]: ...
    def get_annotate_from_class_namespace(obj: Mapping[str, object]) -> AnnotateFunc | None: ...
    @overload
    def get_annotations(
        obj: Any,  # any object with __annotations__ or __annotate__
        *,
        globals: dict[str, object] | None = None,
        locals: Mapping[str, object] | None = None,
        eval_str: bool = False,
        format: Literal[Format.STRING],
    ) -> dict[str, str]: ...
    @overload
    def get_annotations(
        obj: Any,
        *,
        globals: dict[str, object] | None = None,
        locals: Mapping[str, object] | None = None,
        eval_str: bool = False,
        format: Literal[Format.FORWARDREF],
    ) -> dict[str, AnnotationForm | ForwardRef]: ...
    @overload
    def get_annotations(
        obj: Any,
        *,
        globals: dict[str, object] | None = None,
        locals: Mapping[str, object] | None = None,
        eval_str: bool = False,
        format: Format = Format.VALUE,  # noqa: Y011
    ) -> dict[str, AnnotationForm]: ...
    def type_repr(value: object) -> str: ...
    def annotations_to_string(annotations: SupportsItems[str, object]) -> dict[str, str]: ...



from array import array
from collections.abc import Iterator
from typing import Literal, overload

import numpy as np
from numpy.typing import DTypeLike, NDArray

class CoordinateSequence:
    def __init__(self, coords: NDArray[np.float64]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[tuple[float, ...]]: ...
    @overload
    def __getitem__(self, key: int) -> tuple[float, ...]: ...
    @overload
    def __getitem__(self, key: slice) -> list[tuple[float, ...]]: ...
    def __array__(self, dtype: DTypeLike | None = None, copy: Literal[True] | None = None) -> NDArray[np.float64]: ...
    @property
    def xy(self) -> tuple[array[float], array[float]]: ...



from typing import Final

XK_Arabic_comma: Final = 0x5AC
XK_Arabic_semicolon: Final = 0x5BB
XK_Arabic_question_mark: Final = 0x5BF
XK_Arabic_hamza: Final = 0x5C1
XK_Arabic_maddaonalef: Final = 0x5C2
XK_Arabic_hamzaonalef: Final = 0x5C3
XK_Arabic_hamzaonwaw: Final = 0x5C4
XK_Arabic_hamzaunderalef: Final = 0x5C5
XK_Arabic_hamzaonyeh: Final = 0x5C6
XK_Arabic_alef: Final = 0x5C7
XK_Arabic_beh: Final = 0x5C8
XK_Arabic_tehmarbuta: Final = 0x5C9
XK_Arabic_teh: Final = 0x5CA
XK_Arabic_theh: Final = 0x5CB
XK_Arabic_jeem: Final = 0x5CC
XK_Arabic_hah: Final = 0x5CD
XK_Arabic_khah: Final = 0x5CE
XK_Arabic_dal: Final = 0x5CF
XK_Arabic_thal: Final = 0x5D0
XK_Arabic_ra: Final = 0x5D1
XK_Arabic_zain: Final = 0x5D2
XK_Arabic_seen: Final = 0x5D3
XK_Arabic_sheen: Final = 0x5D4
XK_Arabic_sad: Final = 0x5D5
XK_Arabic_dad: Final = 0x5D6
XK_Arabic_tah: Final = 0x5D7
XK_Arabic_zah: Final = 0x5D8
XK_Arabic_ain: Final = 0x5D9
XK_Arabic_ghain: Final = 0x5DA
XK_Arabic_tatweel: Final = 0x5E0
XK_Arabic_feh: Final = 0x5E1
XK_Arabic_qaf: Final = 0x5E2
XK_Arabic_kaf: Final = 0x5E3
XK_Arabic_lam: Final = 0x5E4
XK_Arabic_meem: Final = 0x5E5
XK_Arabic_noon: Final = 0x5E6
XK_Arabic_ha: Final = 0x5E7
XK_Arabic_heh: Final = 0x5E7
XK_Arabic_waw: Final = 0x5E8
XK_Arabic_alefmaksura: Final = 0x5E9
XK_Arabic_yeh: Final = 0x5EA
XK_Arabic_fathatan: Final = 0x5EB
XK_Arabic_dammatan: Final = 0x5EC
XK_Arabic_kasratan: Final = 0x5ED
XK_Arabic_fatha: Final = 0x5EE
XK_Arabic_damma: Final = 0x5EF
XK_Arabic_kasra: Final = 0x5F0
XK_Arabic_shadda: Final = 0x5F1
XK_Arabic_sukun: Final = 0x5F2
XK_Arabic_switch: Final = 0xFF7E



from _typeshed import ConvertibleToInt, Incomplete, SupportsRichComparison
from collections.abc import Callable, Generator
from typing_extensions import Self

from pyasn1.type import base, constraint, namedtype, namedval
from pyasn1.type.tag import TagSet

__all__ = [
    "Integer",
    "Boolean",
    "BitString",
    "OctetString",
    "Null",
    "ObjectIdentifier",
    "Real",
    "Enumerated",
    "SequenceOfAndSetOfBase",
    "SequenceOf",
    "SetOf",
    "SequenceAndSetBase",
    "Sequence",
    "Set",
    "Choice",
    "Any",
    "NoValue",
    "noValue",
]

NoValue = base.NoValue
noValue: NoValue

class Integer(base.SimpleAsn1Type):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    namedValues: namedval.NamedValues
    typeId: int
    def __init__(self, value=..., **kwargs) -> None: ...
    def __and__(self, value): ...
    def __rand__(self, value): ...
    def __or__(self, value): ...
    def __ror__(self, value): ...
    def __xor__(self, value): ...
    def __rxor__(self, value): ...
    def __lshift__(self, value): ...
    def __rshift__(self, value): ...
    def __add__(self, value): ...
    def __radd__(self, value): ...
    def __sub__(self, value): ...
    def __rsub__(self, value): ...
    def __mul__(self, value): ...
    def __rmul__(self, value): ...
    def __mod__(self, value): ...
    def __rmod__(self, value): ...
    # Accepts everything builtins.pow does
    def __pow__(self, value: complex, modulo: int | None = None) -> Self: ...
    def __rpow__(self, value): ...
    def __floordiv__(self, value): ...
    def __rfloordiv__(self, value): ...
    def __truediv__(self, value): ...
    def __rtruediv__(self, value): ...
    def __divmod__(self, value): ...
    def __rdivmod__(self, value): ...
    __hash__ = base.SimpleAsn1Type.__hash__
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __abs__(self): ...
    def __index__(self) -> int: ...
    def __pos__(self): ...
    def __neg__(self): ...
    def __invert__(self): ...
    def __round__(self, n: int = 0): ...
    def __floor__(self): ...
    def __ceil__(self): ...
    def __trunc__(self): ...
    def __lt__(self, value): ...
    def __le__(self, value): ...
    def __eq__(self, value): ...
    def __ne__(self, value): ...
    def __gt__(self, value): ...
    def __ge__(self, value): ...
    def prettyIn(self, value): ...
    def prettyOut(self, value): ...
    def getNamedValues(self): ...

class Boolean(Integer):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    namedValues: namedval.NamedValues
    typeId: int

class SizedInteger(int):
    bitLength: int | None
    leadingZeroBits: int | None
    def setBitLength(self, bitLength): ...
    def __len__(self) -> int: ...

class BitString(base.SimpleAsn1Type):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    namedValues: namedval.NamedValues
    typeId: int
    defaultBinValue: str | base.NoValue
    defaultHexValue: str | base.NoValue
    def __init__(self, value=..., **kwargs) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __len__(self) -> int: ...
    def __getitem__(self, i): ...
    def __iter__(self): ...
    def __reversed__(self): ...
    def __add__(self, value): ...
    def __radd__(self, value): ...
    def __mul__(self, value): ...
    def __rmul__(self, value): ...
    def __lshift__(self, count): ...
    def __rshift__(self, count): ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def asNumbers(self): ...
    def asOctets(self): ...
    def asInteger(self): ...
    def asBinary(self): ...
    @classmethod
    def fromHexString(cls, value, internalFormat: bool = False, prepend: ConvertibleToInt | None = None): ...
    @classmethod
    def fromBinaryString(cls, value, internalFormat: bool = False, prepend: ConvertibleToInt | None = None): ...
    @classmethod
    def fromOctetString(cls, value, internalFormat: bool = False, prepend: ConvertibleToInt | None = None, padding: int = 0): ...
    def prettyIn(self, value): ...

class OctetString(base.SimpleAsn1Type):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    defaultBinValue: str | base.NoValue
    defaultHexValue: str | base.NoValue
    encoding: str
    def __init__(self, value=..., **kwargs) -> None: ...
    def prettyIn(self, value): ...
    def __bytes__(self) -> bytes: ...
    def asOctets(self): ...
    def asNumbers(self): ...
    def prettyOut(self, value): ...
    def prettyPrint(self, scope: int = 0): ...
    @staticmethod
    def fromBinaryString(value): ...
    @staticmethod
    def fromHexString(value): ...
    def __len__(self) -> int: ...
    def __getitem__(self, i): ...
    def __iter__(self): ...
    def __contains__(self, value) -> bool: ...
    def __add__(self, value): ...
    def __radd__(self, value): ...
    def __mul__(self, value): ...
    def __rmul__(self, value): ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __reversed__(self): ...

class Null(OctetString):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    def prettyIn(self, value): ...

class ObjectIdentifier(base.SimpleAsn1Type):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    def __add__(self, other): ...
    def __radd__(self, other): ...
    def asTuple(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, i): ...
    def __iter__(self): ...
    def __contains__(self, value) -> bool: ...
    def index(self, suboid): ...
    def isPrefixOf(self, other): ...
    def prettyIn(self, value): ...
    def prettyOut(self, value): ...

class Real(base.SimpleAsn1Type):
    binEncBase: int | None
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    def prettyIn(self, value): ...
    def prettyPrint(self, scope: int = 0): ...
    @property
    def isPlusInf(self): ...
    @property
    def isMinusInf(self): ...
    @property
    def isInf(self): ...
    def __add__(self, value): ...
    def __radd__(self, value): ...
    def __mul__(self, value): ...
    def __rmul__(self, value): ...
    def __sub__(self, value): ...
    def __rsub__(self, value): ...
    def __mod__(self, value): ...
    def __rmod__(self, value): ...
    # Accepts everything builtins.pow with a float base does
    def __pow__(self, value: complex, modulo: int | None = None) -> Self: ...
    def __rpow__(self, value): ...
    def __truediv__(self, value): ...
    def __rtruediv__(self, value): ...
    def __divmod__(self, value): ...
    def __rdivmod__(self, value): ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __abs__(self): ...
    def __pos__(self): ...
    def __neg__(self): ...
    def __round__(self, n: int = 0): ...
    def __floor__(self): ...
    def __ceil__(self): ...
    def __trunc__(self): ...
    def __lt__(self, value): ...
    def __le__(self, value): ...
    def __eq__(self, value): ...
    def __ne__(self, value): ...
    def __gt__(self, value): ...
    def __ge__(self, value): ...
    def __bool__(self) -> bool: ...
    __hash__ = base.SimpleAsn1Type.__hash__
    def __getitem__(self, idx): ...
    def isPlusInfinity(self): ...
    def isMinusInfinity(self): ...
    def isInfinity(self): ...

class Enumerated(Integer):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    namedValues: namedval.NamedValues

class SequenceOfAndSetOfBase(base.ConstructedAsn1Type):
    componentType: namedtype.NamedTypes | None
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    def __init__(
        self,
        *args,
        componentType: namedtype.NamedTypes | None = ...,
        tagSet: TagSet = ...,
        subtypeSpec: constraint.ConstraintsIntersection = ...,
    ) -> None: ...
    def __getitem__(self, idx): ...
    def __setitem__(self, idx, value) -> None: ...
    def append(self, value) -> None: ...
    def count(self, value): ...
    def extend(self, values) -> None: ...
    def index(self, value, start: int = 0, stop: int | None = None): ...
    def reverse(self) -> None: ...
    def sort(self, key: Callable[[Incomplete], SupportsRichComparison] | None = None, reverse: bool = False) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def getComponentByPosition(self, idx, default=..., instantiate: bool = True): ...
    def setComponentByPosition(
        self, idx, value=..., verifyConstraints: bool = True, matchTags: bool = True, matchConstraints: bool = True
    ): ...
    @property
    def componentTagMap(self): ...
    @property
    def components(self): ...
    def clear(self): ...
    def reset(self): ...
    def prettyPrint(self, scope: int = 0): ...
    def prettyPrintType(self, scope: int = 0): ...
    @property
    def isValue(self): ...
    @property
    def isInconsistent(self): ...

class SequenceOf(SequenceOfAndSetOfBase):
    typeId: int

class SetOf(SequenceOfAndSetOfBase):
    typeId: int

class SequenceAndSetBase(base.ConstructedAsn1Type):
    componentType: namedtype.NamedTypes

    class DynamicNames:
        def __init__(self) -> None: ...
        def __len__(self) -> int: ...
        def __contains__(self, item) -> bool: ...
        def __iter__(self): ...
        def __getitem__(self, item): ...
        def getNameByPosition(self, idx): ...
        def getPositionByName(self, name): ...
        def addField(self, idx) -> None: ...

    def __init__(self, **kwargs) -> None: ...
    def __getitem__(self, idx): ...
    def __setitem__(self, idx, value) -> None: ...
    def __contains__(self, key) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def values(self) -> Generator[Incomplete, None, None]: ...
    def keys(self): ...
    def items(self) -> Generator[Incomplete, None, None]: ...
    def update(self, *iterValue, **mappingValue) -> None: ...
    def clear(self): ...
    def reset(self): ...
    @property
    def components(self): ...
    def getComponentByName(self, name, default=..., instantiate: bool = True): ...
    def setComponentByName(
        self, name, value=..., verifyConstraints: bool = True, matchTags: bool = True, matchConstraints: bool = True
    ): ...
    def getComponentByPosition(self, idx, default=..., instantiate: bool = True): ...
    def setComponentByPosition(
        self, idx, value=..., verifyConstraints: bool = True, matchTags: bool = True, matchConstraints: bool = True
    ): ...
    @property
    def isValue(self): ...
    @property
    def isInconsistent(self): ...
    def prettyPrint(self, scope: int = 0): ...
    def prettyPrintType(self, scope: int = 0): ...
    def setDefaultComponents(self): ...
    def getComponentType(self): ...
    def getNameByPosition(self, idx): ...

class Sequence(SequenceAndSetBase):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    componentType: namedtype.NamedTypes
    typeId: int
    def getComponentTagMapNearPosition(self, idx): ...
    def getComponentPositionNearType(self, tagSet, idx): ...

class Set(SequenceAndSetBase):
    tagSet: TagSet
    componentType: namedtype.NamedTypes
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    def getComponent(self, innerFlag: bool = False): ...
    def getComponentByType(self, tagSet, default=..., instantiate: bool = True, innerFlag: bool = False): ...
    def setComponentByType(
        self,
        tagSet,
        value=...,
        verifyConstraints: bool = True,
        matchTags: bool = True,
        matchConstraints: bool = True,
        innerFlag: bool = False,
    ): ...
    @property
    def componentTagMap(self): ...

class Choice(Set):
    tagSet: TagSet
    componentType: namedtype.NamedTypes
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __contains__(self, key) -> bool: ...
    def __iter__(self): ...
    def values(self) -> Generator[Incomplete, None, None]: ...
    def keys(self) -> Generator[Incomplete, None, None]: ...
    def items(self) -> Generator[Incomplete, None, None]: ...
    def checkConsistency(self) -> None: ...
    def getComponentByPosition(self, idx, default=..., instantiate: bool = True): ...
    def setComponentByPosition(
        self, idx, value=..., verifyConstraints: bool = True, matchTags: bool = True, matchConstraints: bool = True
    ): ...
    @property
    def effectiveTagSet(self): ...
    @property
    def tagMap(self): ...
    def getComponent(self, innerFlag: bool = False): ...
    def getName(self, innerFlag: bool = False): ...
    @property
    def isValue(self): ...
    def clear(self): ...
    def getMinTagSet(self): ...

class Any(OctetString):
    tagSet: TagSet
    subtypeSpec: constraint.ConstraintsIntersection
    typeId: int
    @property
    def tagMap(self): ...



from ...extend.operation import ExtendedOperation
from ...protocol.rfc3062 import PasswdModifyRequestValue, PasswdModifyResponseValue

class ModifyPassword(ExtendedOperation):
    request_name: str
    request_value: PasswdModifyRequestValue
    asn1_spec: PasswdModifyResponseValue
    response_attribute: str
    def config(self) -> None: ...
    def __init__(
        self, connection, user=None, old_password=None, new_password=None, hash_algorithm=None, salt=None, controls=None
    ) -> None: ...
    def populate_result(self) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar

from pony.orm import dbapiprovider
from pony.orm.dbproviders.postgres import (
    PGArrayConverter,
    PGBlobConverter,
    PGColumn,
    PGIntConverter,
    PGProvider,
    PGSchema,
    PGSQLBuilder,
    PGTimedeltaConverter,
    PGTranslator,
)

NoneType: type[None]

class CRColumn(PGColumn):
    auto_template: ClassVar[str]

class CRSchema(PGSchema):
    column_class: ClassVar[type[CRColumn]]

class CRTranslator(PGTranslator): ...
class CRSQLBuilder(PGSQLBuilder): ...

class CRIntConverter(PGIntConverter):
    signed_types: Incomplete
    unsigned_types: Incomplete

class CRBlobConverter(PGBlobConverter):
    def sql_type(converter): ...

class CRTimedeltaConverter(PGTimedeltaConverter): ...

class PGUuidConverter(dbapiprovider.UuidConverter):
    def py2sql(converter, val): ...

class CRArrayConverter(PGArrayConverter):
    array_types: Incomplete

class CRProvider(PGProvider):
    dbschema_cls: ClassVar[type[CRSchema]]
    translator_cls: ClassVar[type[CRTranslator]]
    sqlbuilder_cls: ClassVar[type[CRSQLBuilder]]
    array_converter_cls: ClassVar[type[CRArrayConverter]]

provider_cls = CRProvider



from _typeshed import Incomplete, Unused
from collections.abc import Callable, Mapping
from typing import Any, Final, Literal, Protocol, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias, Unpack

from reportlab.lib.colors import Color
from reportlab.lib.styles import ParagraphStyle, PropertySet, StyleSheet1
from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfgen.textobject import PDFTextObject, _Color
from reportlab.platypus.flowables import Flowable

_T = TypeVar("_T")
_BoolInt: TypeAlias = Literal[0, 1]
_Op: TypeAlias = _SupportsWidthAndExecute | str | float | tuple[str, Unpack[tuple[Any, ...]]]
# NOTE: Output from pyRXP xml parser
_ParsedText: TypeAlias = tuple[str, dict[str, Any], list[_ParsedText], Any] | list[_ParsedText] | str

@type_check_only
class _LineOpHandler(Protocol):
    def start_at(self, x: float, y: float, para: paragraphEngine, canvas: Canvas, textobject: PDFTextObject) -> None: ...
    def end_at(self, x: float, y: float, para: paragraphEngine, canvas: Canvas, textobject: PDFTextObject) -> None: ...

@type_check_only
class _SupportsWidthAndExecute(Protocol):
    def width(self, engine) -> float: ...
    def execute(self, engine, textobject: PDFTextObject, canvas: Canvas) -> object: ...

@type_check_only
class _SimpleStyleKwargs(TypedDict, total=False):
    fontName: str
    fontSize: float
    leading: float
    leftIndent: float
    rightIndent: float
    firstLineIndent: float
    alignment: Literal[0, 1, 2, 4]
    spaceBefore: float
    spaceAfter: float
    bulletFontName: str
    bulletFontSize: float
    bulletIndent: float
    textColor: _Color
    backColor: _Color | None

debug: int
DUMPPROGRAM: int
TOOSMALLSPACE: float

class paragraphEngine:
    TEXT_STATE_VARIABLES: Final[tuple[str, ...]]
    lineOpHandlers: list[_LineOpHandler]
    program: list[_Op]
    indent: float
    baseindent: float
    fontName: str
    fontSize: float
    leading: float
    fontColor: _Color
    x: float
    y: float
    alignment: Literal[0, 1, 2, 4]
    # NOTE: The inner list matches TEXT_STATE_VARIABLES
    textStateStack: list[list[Any]]
    def __init__(self, program: list[_Op] | None = None) -> None: ...
    def pushTextState(self) -> list[Any]: ...
    def popTextState(self) -> None: ...
    def format(
        self, maxwidth: float, maxheight: float, program: list[_Op], leading: float = 0
    ) -> tuple[list[_Op], str, dict[str, Any], float]: ...
    def getState(self) -> dict[str, Any]: ...
    def resetState(self, state: dict[str, Any]) -> None: ...
    def fitLine(
        self, program: list[_Op], totalLength: float
    ) -> tuple[Literal[0, 1], list[_Op], int, float, float, float, Literal[0, 1]]: ...
    def centerAlign(self, line: list[_Op], lineLength: float, maxLength: float) -> list[_Op]: ...
    def rightAlign(self, line: list[_Op], lineLength: float, maxLength: float) -> list[_Op]: ...
    def insertShift(self, line: list[_Op], shift: float) -> list[_Op]: ...
    def justifyAlign(self, line: list[_Op], lineLength: float, maxLength: float) -> list[_Op]: ...
    def shrinkWrap(self, line: list[_Op]) -> list[_Op]: ...
    def cleanProgram(self, line: list[_Op]) -> list[_Op]: ...
    def runOpCodes(self, program: list[_Op], canvas: Canvas, textobject: PDFTextObject) -> dict[str, Any]: ...

def stringLine(line: list[_Op], length: float) -> list[_Op]: ...
def simpleJustifyAlign(line: list[_Op], currentLength: float, maxLength: float) -> list[_Op]: ...
def readBool(text: str) -> _BoolInt: ...
def readAlignment(text: str) -> Literal[0, 1, 2, 4] | None: ...
def readLength(text: str) -> float: ...
@overload
def lengthSequence(s: str, converter: Callable[[str], float] = ...) -> list[float]: ...
@overload
def lengthSequence(s: str, converter: Callable[[str], _T]) -> list[_T]: ...
def readColor(text: str | None) -> Color | None: ...

class StyleAttributeConverters:
    fontSize: list[Callable[[str], float]]
    leading: list[Callable[[str], float]]
    leftIndent: list[Callable[[str], float]]
    rightIndent: list[Callable[[str], float]]
    firstLineIndent: list[Callable[[str], float]]
    alignment: list[Callable[[str], Literal[0, 1, 2, 4] | None]]
    spaceBefore: list[Callable[[str], float]]
    spaceAfter: list[Callable[[str], float]]
    bulletFontSize: list[Callable[[str], float]]
    bulletIndent: list[Callable[[str], float]]
    textColor: list[Callable[[str], Color | None]]
    backColor: list[Callable[[str], Color | None]]

class SimpleStyle:
    name: str
    fontName: str
    fontSize: float
    leading: float
    leftIndent: float
    rightIndent: float
    firstLineIndent: float
    alignment: Literal[0, 1, 2, 4]
    spaceBefore: float
    spaceAfter: float
    bulletFontName: str
    bulletFontSize: float
    bulletIndent: float
    textColor: _Color
    backColor: _Color | None
    # NOTE: We are being generous by allowing PropertySet i.e. ParagraphStyle here
    #       technically SimpleStyle is more strict and doesn't allow string alignments
    def __init__(self, name: str, parent: SimpleStyle | PropertySet | None = None, **kw: Unpack[_SimpleStyleKwargs]) -> None: ...
    def addAttributes(self, dictionary: Mapping[str, str | None]) -> None: ...

DEFAULT_ALIASES: Final[dict[str, str]]

class FastPara(Flowable):
    style: SimpleStyle | ParagraphStyle
    simpletext: str
    lines: list[str] | None
    # NOTE: We are being generous by allowing PropertySet i.e. ParagraphStyle here
    #       technically SimpleStyle is more strict and doesn't allow string alignments
    def __init__(self, style: SimpleStyle | PropertySet, simpletext: str) -> None: ...
    def draw(self) -> None: ...

def defaultContext() -> dict[str, PropertySet]: ...
def buildContext(stylesheet: StyleSheet1 | None = None) -> dict[str, PropertySet]: ...

class Para(Flowable):
    baseindent: float
    context: dict[str, PropertySet]
    parsedText: _ParsedText
    bulletText: str | None
    style1: SimpleStyle | PropertySet
    program: list[_Op]
    formattedProgram: list[_Op]
    remainder: _ParsedText
    state: dict[str, Any]
    bold: _BoolInt
    italic: _BoolInt
    face: str
    size: float
    def __init__(
        self,
        # NOTE: We are being generous by allowing PropertySet i.e. ParagraphStyle here
        #       technically SimpleStyle is more strict and doesn't allow string alignments
        style: SimpleStyle | PropertySet,
        parsedText: tuple[Any, ...] | None = None,
        bulletText: str | None = None,
        state: dict[str, Any] | None = None,
        context: dict[str, PropertySet] | None = None,
        baseindent: float = 0,
    ) -> None: ...
    def draw(self) -> None: ...
    def compileProgram(self, parsedText: _ParsedText, program: list[_Op] | None = None) -> list[_Op]: ...
    def linearize(self, program: list[_Op] | None = None, parsedText: _ParsedText | None = None) -> None: ...
    def compileComponent(self, parsedText: _ParsedText, program: list[_Op]) -> None: ...
    def shiftfont(
        self, program: list[_Op], face: str | None = None, bold: _BoolInt | None = None, italic: _BoolInt | None = None
    ): ...
    def compile_(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_pageNumber(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_b(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_i(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_u(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_sub(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_ul(self, attdict, content, extra, program: list[_Op], tagname: str = "ul") -> None: ...
    def compile_ol(self, attdict, content, extra, program: list[_Op]): ...
    def compile_dl(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_super(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_font(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_a(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_link(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_setLink(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_bullet(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def do_bullet(self, text: str, program: list[_Op]) -> None: ...
    def compile_tt(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_greek(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_evalString(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_name(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_getName(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_seq(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_seqReset(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_seqDefault(self, attdict, content, extra, program: list[_Op]) -> None: ...
    def compile_para(self, attdict, content, extra, program: list[_Op], stylename: str = "para.defaultStyle") -> None: ...

class bulletMaker:
    tagname: Literal["ul", "ol", "dl"]
    style: str
    typ: str
    count: int
    def __init__(self, tagname: Literal["ul", "ol", "dl"], atts, context) -> None: ...
    def makeBullet(self, atts, bl: str | None = None) -> None: ...

class EvalStringObject:
    tagname: str
    attdict: Incomplete
    content: str
    context: Incomplete
    extra: Incomplete
    op: Incomplete
    def __init__(self, attdict, content: str, extra, context) -> None: ...
    def getOp(self, tuple, engine): ...
    def width(self, engine) -> float: ...
    def execute(self, engine, textobject: PDFTextObject, canvas: Canvas) -> None: ...

class SeqObject(EvalStringObject): ...
class NameObject(EvalStringObject): ...
class SeqDefaultObject(NameObject): ...
class SeqResetObject(NameObject): ...
class GetNameObject(EvalStringObject): ...

class PageNumberObject:
    example: str
    def __init__(self, example: str = "XXX") -> None: ...
    def width(self, engine) -> float: ...
    def execute(self, engine, textobject: PDFTextObject, canvas: Canvas) -> None: ...

def EmbedInRml2pdf() -> None: ...
def handleSpecialCharacters(engine, text: str, program: list[_Op] | None = None) -> list[_Op]: ...
def Paragraph(
    text: str,
    style: SimpleStyle | PropertySet,
    bulletText: str | None = None,
    frags: Unused | None = None,
    context: dict[str, PropertySet] | None = None,
) -> Para | FastPara: ...

class UnderLineHandler:
    color: _Color | None
    # NOTE: available after start_at
    xStart: float
    yStart: float
    def __init__(self, color: _Color | None = None) -> None: ...
    def start_at(self, x: float, y: float, para: paragraphEngine, canvas: Canvas, textobject: PDFTextObject) -> None: ...
    def end_at(self, x: float, y: float, para: paragraphEngine, canvas: Canvas, textobject: PDFTextObject) -> None: ...

UNDERLINE: Final[UnderLineHandler]

class HotLink(UnderLineHandler):
    url: str
    def __init__(self, url: str) -> None: ...
    def link(self, rect, canvas: Canvas) -> None: ...

class InternalLink(HotLink): ...

class DefDestination(HotLink):
    defined: _BoolInt

def splitspace(text: str) -> list[str]: ...

testparagraph: str
testparagraph1: str

def test2(canv, testpara) -> None: ...

testlink: Incomplete
test_program: Incomplete

def test() -> None: ...



from _typeshed import Incomplete

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_d_separator", "is_minimal_d_separator", "find_minimal_d_separator"]

@_dispatchable
def is_d_separator(G: DiGraph[_Node], x: _Node | set[_Node], y: _Node | set[_Node], z: _Node | set[_Node]) -> bool: ...
@_dispatchable
def find_minimal_d_separator(G: DiGraph[_Node], x, y, *, included=None, restricted=None) -> set[Incomplete] | None: ...
@_dispatchable
def is_minimal_d_separator(
    G: DiGraph[_Node],
    x: _Node | set[_Node],
    y: _Node | set[_Node],
    z: _Node | set[_Node],
    *,
    included: _Node | set[_Node] | None = None,
    restricted: _Node | set[_Node] | None = None,
) -> bool: ...



from _typeshed import Incomplete

from auth0.types import RequestData

from .rest import RestClient

class AsyncRestClient(RestClient):
    timeout: Incomplete
    def set_session(self, session) -> None: ...
    async def get(self, url: str, params: dict[str, Incomplete] | None = None, headers: dict[str, str] | None = None): ...
    async def post(self, url: str, data: RequestData | None = None, headers: dict[str, str] | None = None): ...
    async def file_post(  # type: ignore[override] # Differs from supertype
        self, url: str, data: dict[str, Incomplete], files: dict[str, Incomplete]
    ): ...
    async def patch(self, url: str, data: RequestData | None = None): ...
    async def put(self, url: str, data: RequestData | None = None): ...
    async def delete(self, url: str, params: dict[str, Incomplete] | None = None, data: RequestData | None = None): ...

class RequestsResponse:
    status_code: int
    headers: Incomplete
    text: str
    def __init__(self, response, text: str) -> None: ...



from _typeshed import Incomplete
from decimal import Decimal
from typing import Final

from braintree.attribute_getter import AttributeGetter
from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.risk_data import RiskData
from braintree.successful_result import SuccessfulResult
from braintree.three_d_secure_info import ThreeDSecureInfo

class CreditCardVerification(AttributeGetter):
    class Status:
        Failed: Final = "failed"
        GatewayRejected: Final = "gateway_rejected"
        ProcessorDeclined: Final = "processor_declined"
        Verified: Final = "verified"

    amount: Decimal | None
    currency_iso_code: Incomplete
    processor_response_code: Incomplete
    processor_response_text: Incomplete
    network_response_code: Incomplete
    network_response_text: Incomplete
    risk_data: RiskData | None
    three_d_secure_info: ThreeDSecureInfo | None
    def __init__(self, gateway, attributes) -> None: ...
    @staticmethod
    def find(verification_id: str) -> CreditCardVerification: ...
    @staticmethod
    def search(*query) -> ResourceCollection: ...
    @staticmethod
    def create(params) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def create_signature() -> list[dict[str, list[str | dict[str, list[str]]]] | dict[str, list[str]] | str]: ...
    def __eq__(self, other: object) -> bool: ...



from typing import Any

use_ssl_context: bool

class Tls:
    ssl_options: Any
    validate: Any
    ca_certs_file: Any
    ca_certs_path: Any
    ca_certs_data: Any
    private_key_password: Any
    version: Any
    private_key_file: Any
    certificate_file: Any
    valid_names: Any
    ciphers: Any
    sni: Any
    def __init__(
        self,
        local_private_key_file=None,
        local_certificate_file=None,
        validate=...,
        version=None,
        ssl_options=None,
        ca_certs_file=None,
        valid_names=None,
        ca_certs_path=None,
        ca_certs_data=None,
        local_private_key_password=None,
        ciphers=None,
        sni=None,
    ) -> None: ...
    def wrap_socket(self, connection, do_handshake: bool = False) -> None: ...
    def start_tls(self, connection): ...

def check_hostname(sock, server_name, additional_names) -> None: ...



from _typeshed import StrPath
from typing import Any

from .formats.base_formats import Format
from .resources import ModelResource

def get_resource_class(model_or_resource_class: str) -> ModelResource[Any]: ...

MIME_TYPE_FORMAT_MAPPING: dict[str, type[Format]]

def get_format_class(format_name: str, file_name: StrPath, encoding: str | None = None) -> Format: ...
def get_default_format_names() -> str: ...



from networkx.classes.graph import Graph, _Node
from networkx.exception import NetworkXError
from networkx.utils.backends import _dispatchable
from networkx.utils.decorators import argmap

__all__ = ["modularity", "partition_quality"]

class NotAPartition(NetworkXError):
    def __init__(self, G: Graph[_Node], collection) -> None: ...

require_partition: argmap

@_dispatchable
def intra_community_edges(G: Graph[_Node], partition): ...
@_dispatchable
def inter_community_edges(G: Graph[_Node], partition): ...
@_dispatchable
def inter_community_non_edges(G: Graph[_Node], partition): ...
@_dispatchable
def modularity(G: Graph[_Node], communities, weight: str | None = "weight", resolution: float = 1): ...
@_dispatchable
def partition_quality(G: Graph[_Node], partition): ...



from _typeshed import SupportsWrite
from collections.abc import Iterable, Iterator, Mapping
from decimal import Decimal
from pathlib import Path
from typing import Any, Final, Literal, overload
from typing_extensions import TypeAlias, deprecated

from qrcode.image.svg import SvgPathImage

# NOTE: Since svgwrite doesn't have any stubs we provide some type aliases, that
#       we can choose to refine in the future, e.g. using a Protocol
_SvgDrawing: TypeAlias = Any  # svgwrite.Drawing
_SvgGroup: TypeAlias = Any  # svgwrite.container.Group

# NOTE: Eventually we may want to consider replacing this with typed dicts, even
#       if that means disallowing non-dict arguments. It should allow anything
#       that is valid to pass into `Address.create`.
_AddressDict: TypeAlias = Mapping[str, str | None]

IBAN_ALLOWED_COUNTRIES: list[str]
QR_IID: dict[str, int]
AMOUNT_REGEX: str
MM_TO_UU: float
BILL_HEIGHT: int
RECEIPT_WIDTH: str
PAYMENT_WIDTH: str
MAX_CHARS_PAYMENT_LINE: int
MAX_CHARS_RECEIPT_LINE: int
A4: tuple[str, str]
LABELS: dict[str, dict[str, str]]
SCISSORS_SVG_PATH: str

class Address:
    @overload
    @classmethod
    def create(cls, *, name: str | None = None, line1: str, line2: str | None = None, country: str | None) -> CombinedAddress: ...
    @overload
    @classmethod
    def create(cls, *, name: str | None = None, line1: str | None = None, line2: str, country: str | None) -> CombinedAddress: ...
    @overload
    @classmethod
    def create(
        cls,
        *,
        name: str,
        street: str | None = None,
        house_num: str | None = None,
        pcode: str,
        city: str,
        country: str | None = None,
    ) -> StructuredAddress: ...
    @staticmethod
    def parse_country(country: str | None) -> str: ...

class CombinedAddress(Address):
    combined: Final = True
    name: str
    line1: str
    line2: str
    country: str
    def __init__(
        self, *, name: str | None = None, line1: str | None = None, line2: str | None = None, country: str | None = None
    ) -> None: ...
    def data_list(self) -> list[str]: ...
    def as_paragraph(self, max_chars: int = 72) -> Iterator[str]: ...

class StructuredAddress(Address):
    combined: Final = False
    name: str
    street: str
    house_num: str
    pcode: str
    city: str
    country: str
    def __init__(
        self,
        *,
        name: str | None = None,
        street: str | None = None,
        house_num: str | None = None,
        pcode: str | None = None,
        city: str | None = None,
        country: str | None = None,
    ) -> None: ...
    def data_list(self) -> list[str]: ...
    def as_paragraph(self, max_chars: int = 72) -> Iterator[str]: ...

class QRBill:
    qr_type: str
    version: str
    coding: int
    allowed_currencies: tuple[Literal["CHF"], Literal["EUR"]]
    font_family: str
    creditor: CombinedAddress | StructuredAddress
    final_creditor: CombinedAddress | StructuredAddress | None
    debtor: CombinedAddress | StructuredAddress | None
    ref_type: str
    reference_number: str | None
    account: str
    account_is_qriban: bool
    amount: str | None
    currency: Literal["CHF", "EUR"]
    additional_information: str
    billing_information: str
    @overload
    def __init__(
        self,
        account: str,
        creditor: _AddressDict,
        final_creditor: None = None,
        amount: Decimal | str | None = None,
        currency: Literal["CHF", "EUR"] = "CHF",
        debtor: _AddressDict | None = None,
        ref_number: None = None,
        reference_number: str | None = None,
        extra_infos: Literal[""] = "",
        additional_information: str = "",
        billing_information: str = "",
        alt_procs: list[str] | tuple[()] | tuple[str] | tuple[str, str] = (),
        language: Literal["en", "de", "fr", "it"] = "en",
        top_line: bool = True,
        payment_line: bool = True,
        font_factor: int = 1,
    ) -> None: ...
    @overload
    @deprecated("ref_number is deprecated and replaced by reference_number")
    def __init__(
        self,
        account: str,
        creditor: _AddressDict,
        final_creditor: None = None,
        amount: Decimal | str | None = None,
        currency: Literal["CHF", "EUR"] = "CHF",
        debtor: _AddressDict | None = None,
        *,
        ref_number: str,
        reference_number: None = None,
        extra_infos: str = "",
        additional_information: str = "",
        billing_information: str = "",
        alt_procs: list[str] | tuple[()] | tuple[str] | tuple[str, str] = (),
        language: Literal["en", "de", "fr", "it"] = "en",
        top_line: bool = True,
        payment_line: bool = True,
        font_factor: int = 1,
    ) -> None: ...
    @overload
    @deprecated("extra_infos is deprecated and replaced by additional_information")
    def __init__(
        self,
        account: str,
        creditor: _AddressDict,
        final_creditor: None = None,
        amount: Decimal | str | None = None,
        currency: Literal["CHF", "EUR"] = "CHF",
        debtor: _AddressDict | None = None,
        ref_number: None = None,
        reference_number: str | None = None,
        *,
        extra_infos: str,
        additional_information: str = "",
        billing_information: str = "",
        alt_procs: list[str] | tuple[()] | tuple[str] | tuple[str, str] = (),
        language: Literal["en", "de", "fr", "it"] = "en",
        top_line: bool = True,
        payment_line: bool = True,
        font_factor: int = 1,
    ) -> None: ...
    @property
    def title_font_info(self) -> dict[str, Any]: ...
    @property
    def font_info(self) -> dict[str, Any]: ...
    def head_font_info(self, part: str | None = None) -> dict[str, Any]: ...
    @property
    def proc_font_info(self) -> dict[str, Any]: ...
    def qr_data(self) -> str: ...
    def qr_image(self) -> SvgPathImage: ...
    def draw_swiss_cross(self, dwg: _SvgDrawing, grp: _SvgGroup, origin: tuple[float, float], size: float) -> None: ...
    def draw_blank_rect(self, dwg: _SvgDrawing, grp: _SvgGroup, x: float, y: float, width: float, height: float) -> None: ...
    def label(self, txt: str) -> str: ...
    def as_svg(self, file_out: str | Path | SupportsWrite[str], full_page: bool = False) -> None: ...
    def transform_to_full_page(self, dwg: _SvgDrawing, bill: _SvgGroup) -> None: ...
    def draw_bill(self, dwg: _SvgDrawing, horiz_scissors: bool = True) -> _SvgGroup: ...

def add_mm(*mms: str | float) -> float: ...
def mm(val: str | float) -> float: ...
def format_ref_number(bill: QRBill) -> str: ...
def format_amount(amount_: str | float) -> str: ...
def wrap_infos(infos: Iterable[str]) -> Iterator[str]: ...
def replace_linebreaks(text: str | None) -> str: ...



from __future__ import annotations

from typing import cast

import grpc
from grpc_reflection.v1alpha.reflection import enable_server_reflection

server = cast(grpc.Server, None)
enable_server_reflection(["foo"], server, None)



from typing import TypedDict, type_check_only

@type_check_only
class _VersionDict(TypedDict):
    version: str

@type_check_only
class _OptionalVersionDict(TypedDict):
    version: str | None

@type_check_only
class _PlatformDict(TypedDict):
    system: str
    release: str

@type_check_only
class _ImplementationDict(_VersionDict):
    name: str

@type_check_only
class _PyOpenSSLDict(_OptionalVersionDict):
    openssl_version: str

@type_check_only
class _InfoDict(TypedDict):
    platform: _PlatformDict
    implementation: _ImplementationDict
    system_ssl: _VersionDict
    using_pyopenssl: bool
    using_charset_normalizer: bool
    pyOpenSSL: _PyOpenSSLDict
    urllib3: _VersionDict
    chardet: _OptionalVersionDict
    charset_normalizer: _OptionalVersionDict
    cryptography: _VersionDict
    idna: _VersionDict
    requests: _VersionDict

def info() -> _InfoDict: ...
def main() -> None: ...



from braintree.graphql.types.payment_options import PaymentOptions
from braintree.graphql.types.payment_recommendation import PaymentRecommendation

class CustomerRecommendations:
    payment_options: list[PaymentOptions]
    payment_recommendations: list[PaymentRecommendation]
    def __init__(self, payment_recommendations: list[PaymentRecommendation] | None = None) -> None: ...



from networkx.classes.graph import Graph, _Edge, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["betweenness_centrality", "edge_betweenness_centrality"]

@_dispatchable
def betweenness_centrality(
    G: Graph[_Node],
    k: int | None = None,
    normalized: bool | None = True,
    weight: str | None = None,
    endpoints: bool | None = False,
    seed: int | RandomState | None = None,
) -> dict[_Node, float]: ...
@_dispatchable
def edge_betweenness_centrality(
    G: Graph[_Node],
    k: int | None = None,
    normalized: bool | None = True,
    weight: str | None = None,
    seed: int | RandomState | None = None,
) -> dict[_Edge[_Node], float]: ...



from _typeshed import Incomplete

class TokenValidator:
    TOKEN_TYPE: str
    realm: Incomplete
    extra_attributes: Incomplete
    def __init__(self, realm=None, **extra_attributes) -> None: ...
    @staticmethod
    def scope_insufficient(token_scopes, required_scopes): ...
    def authenticate_token(self, token_string) -> None: ...
    def validate_request(self, request) -> None: ...
    def validate_token(self, token, scopes, request) -> None: ...

class ResourceProtector:
    def __init__(self) -> None: ...
    def register_token_validator(self, validator: TokenValidator): ...
    def get_token_validator(self, token_type): ...
    def parse_request_authorization(self, request): ...
    def validate_request(self, scopes, request, **kwargs): ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Github(VaultApiBase):
    def configure(self, organization, base_url=None, ttl=None, max_ttl=None, mount_point="github"): ...
    def read_configuration(self, mount_point="github"): ...
    def map_team(self, team_name, policies=None, mount_point="github"): ...
    def read_team_mapping(self, team_name, mount_point="github"): ...
    def map_user(self, user_name, policies=None, mount_point="github"): ...
    def read_user_mapping(self, user_name, mount_point="github"): ...
    def login(self, token, use_token: bool = True, mount_point="github"): ...



import datetime

from ..core import WesternCalendar

class Guernsey(WesternCalendar):
    def get_spring_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_early_may_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_summer_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_liberation_day(self, year: int) -> tuple[datetime.date, str]: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Any

RED: tuple[int, int, int]
ORANGE: tuple[int, int, int]
YELLOW: tuple[int, int, int]
GREEN: tuple[int, int, int]
LIGHT_BLUE: tuple[int, int, int]
BLUE: tuple[int, int, int]
PINK: tuple[int, int, int]
PURPLE: tuple[int, int, int]
WHITE: tuple[int, int, int]

class Nanoleaf:
    ip: str
    print_errors: bool
    url: str
    auth_token: str
    already_registered: bool
    def __init__(self, ip: str, auth_token: str | None = None, print_errors: bool = False) -> None: ...
    def create_auth_token(self) -> str | None: ...
    def delete_auth_token(self, auth_token: str) -> bool: ...
    def check_connection(self) -> None: ...
    def get_info(self) -> dict[str, Incomplete]: ...
    def get_name(self) -> str: ...
    def get_auth_token(self) -> str | None: ...
    def get_ids(self) -> list[int]: ...
    @staticmethod
    def get_custom_base_effect(anim_type: str = "custom", loop: bool = True) -> dict[str, Incomplete]: ...
    def power_off(self) -> bool: ...
    def power_on(self) -> bool: ...
    def get_power(self) -> bool: ...
    def toggle_power(self) -> bool: ...
    def set_color(self, rgb: tuple[int, int, int]) -> bool: ...
    def set_brightness(self, brightness: int, duration: int = 0) -> bool: ...
    def increment_brightness(self, brightness: int) -> bool: ...
    def get_brightness(self) -> int: ...
    def identify(self) -> bool: ...
    def set_hue(self, value: int) -> bool: ...
    def increment_hue(self, value: int) -> bool: ...
    def get_hue(self) -> int: ...
    def set_saturation(self, value: int) -> bool: ...
    def increment_saturation(self, value: int) -> bool: ...
    def get_saturation(self) -> int: ...
    def set_color_temp(self, value: int) -> bool: ...
    def increment_color_temp(self, value: int) -> bool: ...
    def get_color_temp(self) -> int: ...
    def get_color_mode(self) -> str: ...
    def get_current_effect(self) -> str: ...
    def set_effect(self, effect_name: str) -> bool: ...
    def list_effects(self) -> list[str]: ...
    def write_effect(self, effect_dict: dict[str, Incomplete]) -> bool: ...
    def effect_exists(self, effect_name: str) -> bool: ...
    def pulsate(self, rgb: tuple[int, int, int], speed: float = 1) -> bool: ...
    def flow(self, rgb_list: list[tuple[int, int, int]], speed: float = 1) -> bool: ...
    def spectrum(self, speed: float = 1) -> bool: ...
    def enable_extcontrol(self) -> bool: ...
    def get_layout(self) -> dict[str, Incomplete]: ...
    def register_event(self, func: Callable[[dict[str, Incomplete]], Any], event_types: list[int]) -> None: ...

class NanoleafRegistrationError(Exception):
    def __init__(self) -> None: ...

class NanoleafConnectionError(Exception):
    def __init__(self) -> None: ...

class NanoleafEffectCreationError(Exception): ...



import subprocess
from _typeshed import StrOrBytesPath
from collections.abc import Callable
from os import _Environ
from typing import AnyStr

from .spawnbase import SpawnBase, _Logfile

class PopenSpawn(SpawnBase[AnyStr]):
    proc: subprocess.Popen[AnyStr]
    closed: bool
    def __init__(
        self,
        cmd,
        timeout: float | None = 30,
        maxread: int = 2000,
        searchwindowsize: int | None = None,
        logfile: _Logfile | None = None,
        cwd: StrOrBytesPath | None = None,
        env: _Environ[str] | None = None,
        encoding: str | None = None,
        codec_errors: str = "strict",
        preexec_fn: Callable[[], None] | None = None,
    ) -> None: ...
    flag_eof: bool
    def read_nonblocking(self, size, timeout): ...  # type: ignore[override]
    def write(self, s) -> None: ...
    def writelines(self, sequence) -> None: ...
    def send(self, s): ...
    def sendline(self, s: str = ""): ...
    terminated: bool
    def wait(self): ...
    def kill(self, sig) -> None: ...
    def sendeof(self) -> None: ...



from __future__ import annotations

import copy
import sys
from typing import Generic, TypeVar
from typing_extensions import Self, assert_type


class ReplaceableClass:
    def __init__(self, val: int) -> None:
        self.val = val

    def __replace__(self, val: int) -> Self:
        cpy = copy.copy(self)
        cpy.val = val
        return cpy


if sys.version_info >= (3, 13):
    obj = ReplaceableClass(42)
    cpy = copy.replace(obj, val=23)
    assert_type(cpy, ReplaceableClass)


_T_co = TypeVar("_T_co", covariant=True)


class Box(Generic[_T_co]):
    def __init__(self, value: _T_co, /) -> None:
        self.value = value

    def __replace__(self, value: str) -> Box[str]:
        return Box(value)


if sys.version_info >= (3, 13):
    box1: Box[int] = Box(42)
    box2 = copy.replace(box1, val="spam")
    assert_type(box2, Box[str])



from authlib.common.errors import AuthlibBaseError

class OAuthError(AuthlibBaseError):
    error: str

class MissingRequestTokenError(OAuthError):
    error: str

class MissingTokenError(OAuthError):
    error: str

class TokenExpiredError(OAuthError):
    error: str

class InvalidTokenError(OAuthError):
    error: str

class UnsupportedTokenTypeError(OAuthError):
    error: str

class MismatchingStateError(OAuthError):
    error: str
    description: str



from collections.abc import Iterator
from functools import cached_property
from typing import Any
from typing_extensions import Self

from networkx.classes.coreviews import AdjacencyView
from networkx.classes.graph import Graph, _Node
from networkx.classes.reportviews import (
    DiDegreeView,
    InDegreeView,
    InEdgeView,
    InMultiDegreeView,
    InMultiEdgeView,
    OutDegreeView,
    OutEdgeView,
    OutMultiDegreeView,
)

__all__ = ["DiGraph"]

# NOTE: Graph subclasses relationships are so complex
# we're only overriding methods that differ in signature from the base classes
# to use inheritance to our advantage and reduce complexity
class DiGraph(Graph[_Node]):
    @cached_property
    def succ(self) -> AdjacencyView[_Node, _Node, dict[str, Any]]: ...
    @cached_property
    def pred(self) -> AdjacencyView[_Node, _Node, dict[str, Any]]: ...
    def has_successor(self, u: _Node, v: _Node) -> bool: ...
    def has_predecessor(self, u: _Node, v: _Node) -> bool: ...
    def successors(self, n: _Node) -> Iterator[_Node]: ...

    neighbors = successors

    def predecessors(self, n: _Node) -> Iterator[_Node]: ...
    @cached_property
    def edges(self) -> OutEdgeView[_Node]: ...
    @cached_property
    def out_edges(self) -> OutEdgeView[_Node]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def in_edges(self) -> InEdgeView[_Node] | InMultiEdgeView[_Node]: ...
    @cached_property
    def degree(self) -> DiDegreeView[_Node]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def in_degree(self) -> InDegreeView[_Node] | InMultiDegreeView[_Node]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def out_degree(self) -> OutDegreeView[_Node] | OutMultiDegreeView[_Node]: ...
    def to_undirected(self, reciprocal: bool = False, as_view: bool = False) -> Graph[_Node]: ...  # type: ignore[override] # Has an additional `reciprocal` keyword argument
    def reverse(self, copy: bool = True) -> Self: ...



from _typeshed import Incomplete
from typing import Final

from braintree.configuration import Configuration
from braintree.environment import Environment

class Http:
    class ContentType:
        Xml: Final = "application/xml"
        Multipart: Final = "multipart/form-data"
        Json: Final = "application/json"

    @staticmethod
    def is_error_status(status: int) -> bool: ...
    @staticmethod
    def raise_exception_from_status(status: int, message: str | None = None) -> None: ...
    config: Configuration
    environment: Environment
    def __init__(self, config: Configuration, environment: Environment | None = None) -> None: ...
    def post(self, path: str, params: dict[str, Incomplete] | None = None): ...
    def delete(self, path: str): ...
    def get(self, path: str): ...
    def put(self, path: str, params: dict[str, Incomplete] | None = None): ...
    def post_multipart(self, path: str, files, params: dict[str, Incomplete] | None = None): ...
    def http_do(
        self, http_verb: str, path: str, headers: dict[str, Incomplete], request_body: str | tuple[str, Incomplete] | None
    ) -> list[int | str]: ...
    def handle_exception(self, exception: IOError) -> None: ...



from types import TracebackType
from typing import Generic, TypeVar, final, overload
from typing_extensions import TypeAlias

from gevent.event import _ValueSource
from gevent.hub import Hub
from greenlet import greenlet as greenlet_t

__all__ = ["Waiter"]

_T = TypeVar("_T")
# this is annoying, it's due to them using *throw args, rather than just storing them in standardized form
_ThrowArgs: TypeAlias = (
    tuple[()]
    | tuple[BaseException]
    | tuple[BaseException, None]
    | tuple[BaseException, None, TracebackType | None]
    | tuple[type[BaseException]]
    | tuple[type[BaseException], BaseException | object]
    | tuple[type[BaseException], BaseException | object, TracebackType | None]
)

class Waiter(Generic[_T]):
    __slots__ = ["hub", "greenlet", "value", "_exception"]
    @property
    def hub(self) -> Hub: ...  # readonly in Cython
    @property
    def greenlet(self) -> greenlet_t | None: ...  # readonly in Cython
    @property
    def value(self) -> _T | None: ...  # readonly in Cython
    def __init__(self, hub: Hub | None = None) -> None: ...
    def clear(self) -> None: ...
    def ready(self) -> bool: ...
    def successful(self) -> bool: ...
    @property
    def exc_info(self) -> _ThrowArgs | None: ...
    def switch(self, value: _T) -> None: ...
    @overload
    def throw(self, typ: type[BaseException], val: BaseException | object = None, tb: TracebackType | None = None, /) -> None: ...
    @overload
    def throw(self, typ: BaseException = ..., val: None = None, tb: TracebackType | None = None, /) -> None: ...
    def get(self) -> _T: ...
    def __call__(self, source: _ValueSource[_T]) -> None: ...

@final
class MultipleWaiter(Waiter[_T]):
    __slots__ = ["_values"]



from pathlib import Path
from typing import Final

# TODO: Use base path relative to this file. Currently, ts_utils gets
# installed into the user's virtual env, so we can't determine the path
# to typeshed. Installing ts_utils editable would solve that, see
# https://github.com/python/typeshed/pull/12806.
TS_BASE_PATH: Final = Path()
STDLIB_PATH: Final = TS_BASE_PATH / "stdlib"
STUBS_PATH: Final = TS_BASE_PATH / "stubs"

PYPROJECT_PATH: Final = TS_BASE_PATH / "pyproject.toml"
REQUIREMENTS_PATH: Final = TS_BASE_PATH / "requirements-tests.txt"
GITIGNORE_PATH: Final = TS_BASE_PATH / ".gitignore"
PYRIGHT_CONFIG: Final = TS_BASE_PATH / "pyrightconfig.stricter.json"

TESTS_DIR: Final = "@tests"
TEST_CASES_DIR: Final = "test_cases"


def distribution_path(distribution_name: str) -> Path:
    """Return the path to the directory of a third-party distribution."""
    return STUBS_PATH / distribution_name


def tests_path(distribution_name: str) -> Path:
    if distribution_name == "stdlib":
        return STDLIB_PATH / TESTS_DIR
    else:
        return STUBS_PATH / distribution_name / TESTS_DIR


def test_cases_path(distribution_name: str) -> Path:
    return tests_path(distribution_name) / TEST_CASES_DIR


def allowlists_path(distribution_name: str) -> Path:
    if distribution_name == "stdlib":
        return tests_path("stdlib") / "stubtest_allowlists"
    else:
        return tests_path(distribution_name)



from _typeshed import Incomplete
from collections.abc import Generator, Iterable, Iterator
from typing import overload

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "find_cliques",
    "find_cliques_recursive",
    "make_max_clique_graph",
    "make_clique_bipartite",
    "node_clique_number",
    "number_of_cliques",
    "enumerate_all_cliques",
    "max_weight_clique",
]

@_dispatchable
def enumerate_all_cliques(G: Graph[_Node]) -> Generator[list[_Node], None, None]: ...
@_dispatchable
def find_cliques(G: Graph[_Node], nodes: Iterable[Incomplete] | None = None) -> Generator[list[_Node], None, None]: ...
@_dispatchable
def find_cliques_recursive(G: Graph[_Node], nodes: Iterable[Incomplete] | None = None) -> Iterator[list[_Node]]: ...
@_dispatchable
def make_max_clique_graph(G: Graph[_Node], create_using: Graph[_Node] | None = None) -> Graph[_Node]: ...
@_dispatchable
def make_clique_bipartite(
    G: Graph[_Node], fpos: bool | None = None, create_using: Graph[_Node] | None = None, name=None
) -> Graph[_Node]: ...
@overload
def node_clique_number(
    G: Graph[_Node], nodes=None, cliques: Iterable[Incomplete] | None = None, separate_nodes=False
) -> dict[_Node, int]: ...
@overload
def node_clique_number(G: Graph[_Node], nodes=None, cliques: Iterable[Incomplete] | None = None, separate_nodes=False) -> int: ...
def number_of_cliques(G: Graph[_Node], nodes=None, cliques=None) -> int | dict[Incomplete, Incomplete]: ...
@_dispatchable
def max_weight_clique(G: Graph[_Node], weight="weight") -> tuple[list[Incomplete], int]: ...

class MaxWeightClique:
    G: Graph[Incomplete]
    incumbent_nodes: list[Incomplete]
    incumbent_weight: int
    node_weights: dict[Incomplete, int]
    def __init__(self, G: Graph[_Node], weight): ...
    def update_incumbent_if_improved(self, C, C_weight): ...
    def greedily_find_independent_set(self, P): ...
    def find_branching_nodes(self, P, target): ...
    def expand(self, C, C_weight, P): ...
    def find_max_weight_clique(self): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["triangles", "average_clustering", "clustering", "transitivity", "square_clustering", "generalized_degree"]

@_dispatchable
def triangles(G: Graph[_Node], nodes=None) -> int | dict[Incomplete, int]: ...
@_dispatchable
def average_clustering(
    G: Graph[_Node], nodes: Iterable[_Node] | None = None, weight: str | None = None, count_zeros: bool = True
) -> float: ...
@_dispatchable
def clustering(G: Graph[_Node], nodes=None, weight: str | None = None) -> float | int | dict[Incomplete, float | int]: ...
@_dispatchable
def transitivity(G: Graph[_Node]) -> float: ...
@_dispatchable
def square_clustering(G: Graph[_Node], nodes: Iterable[_Node] | None = None) -> float | int | dict[Incomplete, float | int]: ...
@_dispatchable
def generalized_degree(G: Graph[_Node], nodes: Iterable[_Node] | None = None): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.graph_pb2
import tensorflow.core.framework.op_def_pb2
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2
import tensorflow.core.protobuf.saved_object_graph_pb2
import tensorflow.core.protobuf.saver_pb2
import tensorflow.core.protobuf.struct_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class MetaGraphDef(google.protobuf.message.Message):
    """Protocol buffer containing the following which are necessary to restart
    training, run inference. It can be used to serialize/de-serialize memory
    objects necessary for running computation in a graph when crossing the
    process boundary. It can be used for long term storage of graphs,
    cross-language execution of graphs, etc.
      MetaInfoDef
      GraphDef
      SaverDef
      CollectionDef
      TensorInfo
      SignatureDef
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetaInfoDef(google.protobuf.message.Message):
        """Meta information regarding the graph to be exported.  To be used by users
        of this protocol buffer to encode information regarding their meta graph.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class FunctionAliasesEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            value: builtins.str
            def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
            def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

        META_GRAPH_VERSION_FIELD_NUMBER: builtins.int
        STRIPPED_OP_LIST_FIELD_NUMBER: builtins.int
        ANY_INFO_FIELD_NUMBER: builtins.int
        TAGS_FIELD_NUMBER: builtins.int
        TENSORFLOW_VERSION_FIELD_NUMBER: builtins.int
        TENSORFLOW_GIT_VERSION_FIELD_NUMBER: builtins.int
        STRIPPED_DEFAULT_ATTRS_FIELD_NUMBER: builtins.int
        FUNCTION_ALIASES_FIELD_NUMBER: builtins.int
        meta_graph_version: builtins.str
        """User specified Version string. Can be the name of the model and revision,
        steps this model has been trained to, etc.
        """
        tensorflow_version: builtins.str
        """The __version__ string of the tensorflow build used to write this graph.
        This will be populated by the framework, which will overwrite any user
        supplied value.
        """
        tensorflow_git_version: builtins.str
        """The __git_version__ string of the tensorflow build used to write this
        graph. This will be populated by the framework, which will overwrite any
        user supplied value.
        """
        stripped_default_attrs: builtins.bool
        """A flag to denote whether default-valued attrs have been stripped from
        the nodes in this graph_def.
        """
        @property
        def stripped_op_list(self) -> tensorflow.core.framework.op_def_pb2.OpList:
            """A copy of the OpDefs used by the producer of this graph_def.
            Descriptions and Ops not used in graph_def are stripped out.
            """

        @property
        def any_info(self) -> google.protobuf.any_pb2.Any:
            """A serialized protobuf. Can be the time this meta graph is created, or
            modified, or name of the model.
            """

        @property
        def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
            """User supplied tag(s) on the meta_graph and included graph_def.

            MetaGraphDefs should be tagged with their capabilities or use-cases.
            Examples: "train", "serve", "gpu", "tpu", etc.
            These tags enable loaders to access the MetaGraph(s) appropriate for a
            specific use-case or runtime environment.
            """

        @property
        def function_aliases(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
            """FunctionDef name to aliases mapping."""

        def __init__(
            self,
            *,
            meta_graph_version: builtins.str | None = ...,
            stripped_op_list: tensorflow.core.framework.op_def_pb2.OpList | None = ...,
            any_info: google.protobuf.any_pb2.Any | None = ...,
            tags: collections.abc.Iterable[builtins.str] | None = ...,
            tensorflow_version: builtins.str | None = ...,
            tensorflow_git_version: builtins.str | None = ...,
            stripped_default_attrs: builtins.bool | None = ...,
            function_aliases: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        ) -> None: ...
        def HasField(
            self, field_name: typing.Literal["any_info", b"any_info", "stripped_op_list", b"stripped_op_list"]
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "any_info",
                b"any_info",
                "function_aliases",
                b"function_aliases",
                "meta_graph_version",
                b"meta_graph_version",
                "stripped_default_attrs",
                b"stripped_default_attrs",
                "stripped_op_list",
                b"stripped_op_list",
                "tags",
                b"tags",
                "tensorflow_git_version",
                b"tensorflow_git_version",
                "tensorflow_version",
                b"tensorflow_version",
            ],
        ) -> None: ...

    @typing.final
    class CollectionDefEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___CollectionDef: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___CollectionDef | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class SignatureDefEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___SignatureDef: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___SignatureDef | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    META_INFO_DEF_FIELD_NUMBER: builtins.int
    GRAPH_DEF_FIELD_NUMBER: builtins.int
    SAVER_DEF_FIELD_NUMBER: builtins.int
    COLLECTION_DEF_FIELD_NUMBER: builtins.int
    SIGNATURE_DEF_FIELD_NUMBER: builtins.int
    ASSET_FILE_DEF_FIELD_NUMBER: builtins.int
    OBJECT_GRAPH_DEF_FIELD_NUMBER: builtins.int
    @property
    def meta_info_def(self) -> global___MetaGraphDef.MetaInfoDef: ...
    @property
    def graph_def(self) -> tensorflow.core.framework.graph_pb2.GraphDef:
        """GraphDef."""

    @property
    def saver_def(self) -> tensorflow.core.protobuf.saver_pb2.SaverDef:
        """SaverDef."""

    @property
    def collection_def(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___CollectionDef]:
        """collection_def: Map from collection name to collections.
        See CollectionDef section for details.
        """

    @property
    def signature_def(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___SignatureDef]:
        """signature_def: Map from user supplied key for a signature to a single
        SignatureDef.
        """

    @property
    def asset_file_def(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AssetFileDef]:
        """Asset file def to be used with the defined graph."""

    @property
    def object_graph_def(self) -> tensorflow.core.protobuf.saved_object_graph_pb2.SavedObjectGraph:
        """Extra information about the structure of functions and stateful objects."""

    def __init__(
        self,
        *,
        meta_info_def: global___MetaGraphDef.MetaInfoDef | None = ...,
        graph_def: tensorflow.core.framework.graph_pb2.GraphDef | None = ...,
        saver_def: tensorflow.core.protobuf.saver_pb2.SaverDef | None = ...,
        collection_def: collections.abc.Mapping[builtins.str, global___CollectionDef] | None = ...,
        signature_def: collections.abc.Mapping[builtins.str, global___SignatureDef] | None = ...,
        asset_file_def: collections.abc.Iterable[global___AssetFileDef] | None = ...,
        object_graph_def: tensorflow.core.protobuf.saved_object_graph_pb2.SavedObjectGraph | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "graph_def",
            b"graph_def",
            "meta_info_def",
            b"meta_info_def",
            "object_graph_def",
            b"object_graph_def",
            "saver_def",
            b"saver_def",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "asset_file_def",
            b"asset_file_def",
            "collection_def",
            b"collection_def",
            "graph_def",
            b"graph_def",
            "meta_info_def",
            b"meta_info_def",
            "object_graph_def",
            b"object_graph_def",
            "saver_def",
            b"saver_def",
            "signature_def",
            b"signature_def",
        ],
    ) -> None: ...

global___MetaGraphDef = MetaGraphDef

@typing.final
class CollectionDef(google.protobuf.message.Message):
    """CollectionDef should cover most collections.
    To add a user-defined collection, do one of the following:
    1. For simple data types, such as string, int, float:
         tf.add_to_collection("your_collection_name", your_simple_value)
       strings will be stored as bytes_list.

    2. For Protobuf types, there are three ways to add them:
       1) tf.add_to_collection("your_collection_name",
            your_proto.SerializeToString())

          collection_def {
            key: "user_defined_bytes_collection"
            value {
              bytes_list {
                value: "queue_name: \\"test_queue\\"\\n"
              }
            }
          }

     or

       2) tf.add_to_collection("your_collection_name", str(your_proto))

          collection_def {
            key: "user_defined_string_collection"
            value {
             bytes_list {
                value: "\\n\\ntest_queue"
              }
            }
          }

     or

       3) any_buf = any_pb2.Any()
          tf.add_to_collection("your_collection_name",
            any_buf.Pack(your_proto))

          collection_def {
            key: "user_defined_any_collection"
            value {
              any_list {
                value {
                  type_url: "type.googleapis.com/tensorflow.QueueRunnerDef"
                  value: "\\n\\ntest_queue"
                }
              }
            }
          }

    3. For Python objects, implement to_proto() and from_proto(), and register
       them in the following manner:
       ops.register_proto_function("your_collection_name",
                                   proto_type,
                                   to_proto=YourPythonObject.to_proto,
                                   from_proto=YourPythonObject.from_proto)
       These functions will be invoked to serialize and de-serialize the
       collection. For example,
       ops.register_proto_function(ops.GraphKeys.GLOBAL_VARIABLES,
                                   proto_type=variable_pb2.VariableDef,
                                   to_proto=Variable.to_proto,
                                   from_proto=Variable.from_proto)
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class NodeList(google.protobuf.message.Message):
        """NodeList is used for collecting nodes in graph. For example
        collection_def {
          key: "summaries"
          value {
            node_list {
              value: "input_producer/ScalarSummary:0"
              value: "shuffle_batch/ScalarSummary:0"
              value: "ImageSummary:0"
            }
          }
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUE_FIELD_NUMBER: builtins.int
        @property
        def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        def __init__(self, *, value: collections.abc.Iterable[builtins.str] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

    @typing.final
    class BytesList(google.protobuf.message.Message):
        """BytesList is used for collecting strings and serialized protobufs. For
        example:
        collection_def {
          key: "trainable_variables"
          value {
            bytes_list {
              value: "\\n\\017conv1/weights:0\\022\\024conv1/weights/Assign
                     \\032\\024conv1/weights/read:0"
              value: "\\n\\016conv1/biases:0\\022\\023conv1/biases/Assign\\032
                     \\023conv1/biases/read:0"
            }
          }
        }
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUE_FIELD_NUMBER: builtins.int
        @property
        def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
        def __init__(self, *, value: collections.abc.Iterable[builtins.bytes] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

    @typing.final
    class Int64List(google.protobuf.message.Message):
        """Int64List is used for collecting int, int64 and long values."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUE_FIELD_NUMBER: builtins.int
        @property
        def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(self, *, value: collections.abc.Iterable[builtins.int] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

    @typing.final
    class FloatList(google.protobuf.message.Message):
        """FloatList is used for collecting float values."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUE_FIELD_NUMBER: builtins.int
        @property
        def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
        def __init__(self, *, value: collections.abc.Iterable[builtins.float] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

    @typing.final
    class AnyList(google.protobuf.message.Message):
        """AnyList is used for collecting Any protos."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUE_FIELD_NUMBER: builtins.int
        @property
        def value(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]: ...
        def __init__(self, *, value: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["value", b"value"]) -> None: ...

    NODE_LIST_FIELD_NUMBER: builtins.int
    BYTES_LIST_FIELD_NUMBER: builtins.int
    INT64_LIST_FIELD_NUMBER: builtins.int
    FLOAT_LIST_FIELD_NUMBER: builtins.int
    ANY_LIST_FIELD_NUMBER: builtins.int
    @property
    def node_list(self) -> global___CollectionDef.NodeList: ...
    @property
    def bytes_list(self) -> global___CollectionDef.BytesList: ...
    @property
    def int64_list(self) -> global___CollectionDef.Int64List: ...
    @property
    def float_list(self) -> global___CollectionDef.FloatList: ...
    @property
    def any_list(self) -> global___CollectionDef.AnyList: ...
    def __init__(
        self,
        *,
        node_list: global___CollectionDef.NodeList | None = ...,
        bytes_list: global___CollectionDef.BytesList | None = ...,
        int64_list: global___CollectionDef.Int64List | None = ...,
        float_list: global___CollectionDef.FloatList | None = ...,
        any_list: global___CollectionDef.AnyList | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "any_list",
            b"any_list",
            "bytes_list",
            b"bytes_list",
            "float_list",
            b"float_list",
            "int64_list",
            b"int64_list",
            "kind",
            b"kind",
            "node_list",
            b"node_list",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "any_list",
            b"any_list",
            "bytes_list",
            b"bytes_list",
            "float_list",
            b"float_list",
            "int64_list",
            b"int64_list",
            "kind",
            b"kind",
            "node_list",
            b"node_list",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["kind", b"kind"]
    ) -> typing.Literal["node_list", "bytes_list", "int64_list", "float_list", "any_list"] | None: ...

global___CollectionDef = CollectionDef

@typing.final
class TensorInfo(google.protobuf.message.Message):
    """Information about a Tensor necessary for feeding or retrieval."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class CooSparse(google.protobuf.message.Message):
        """For sparse tensors, The COO encoding stores a triple of values, indices,
        and shape.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUES_TENSOR_NAME_FIELD_NUMBER: builtins.int
        INDICES_TENSOR_NAME_FIELD_NUMBER: builtins.int
        DENSE_SHAPE_TENSOR_NAME_FIELD_NUMBER: builtins.int
        values_tensor_name: builtins.str
        """The shape of the values Tensor is [?].  Its dtype must be the dtype of
        the SparseTensor as a whole, given in the enclosing TensorInfo.
        """
        indices_tensor_name: builtins.str
        """The indices Tensor must have dtype int64 and shape [?, ?]."""
        dense_shape_tensor_name: builtins.str
        """The dynamic logical shape represented by the SparseTensor is recorded in
        the Tensor referenced here.  It must have dtype int64 and shape [?].
        """
        def __init__(
            self,
            *,
            values_tensor_name: builtins.str | None = ...,
            indices_tensor_name: builtins.str | None = ...,
            dense_shape_tensor_name: builtins.str | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "dense_shape_tensor_name",
                b"dense_shape_tensor_name",
                "indices_tensor_name",
                b"indices_tensor_name",
                "values_tensor_name",
                b"values_tensor_name",
            ],
        ) -> None: ...

    @typing.final
    class CompositeTensor(google.protobuf.message.Message):
        """Generic encoding for composite tensors."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_SPEC_FIELD_NUMBER: builtins.int
        COMPONENTS_FIELD_NUMBER: builtins.int
        @property
        def type_spec(self) -> tensorflow.core.protobuf.struct_pb2.TypeSpecProto:
            """The serialized TypeSpec for the composite tensor."""

        @property
        def components(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TensorInfo]:
            """A TensorInfo for each flattened component tensor."""

        def __init__(
            self,
            *,
            type_spec: tensorflow.core.protobuf.struct_pb2.TypeSpecProto | None = ...,
            components: collections.abc.Iterable[global___TensorInfo] | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["type_spec", b"type_spec"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["components", b"components", "type_spec", b"type_spec"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    COO_SPARSE_FIELD_NUMBER: builtins.int
    COMPOSITE_TENSOR_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    TENSOR_SHAPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """For dense `Tensor`s, the name of the tensor in the graph."""
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def coo_sparse(self) -> global___TensorInfo.CooSparse:
        """There are many possible encodings of sparse matrices
        (https://en.wikipedia.org/wiki/Sparse_matrix).  Currently, TensorFlow
        uses only the COO encoding.  This is supported and documented in the
        SparseTensor Python class.
        """

    @property
    def composite_tensor(self) -> global___TensorInfo.CompositeTensor:
        """Generic encoding for CompositeTensors."""

    @property
    def tensor_shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """The static shape should be recorded here, to the extent that it can
        be known in advance.  In the case of a SparseTensor, this field describes
        the logical shape of the represented tensor (aka dense_shape).
        """

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        coo_sparse: global___TensorInfo.CooSparse | None = ...,
        composite_tensor: global___TensorInfo.CompositeTensor | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        tensor_shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "composite_tensor",
            b"composite_tensor",
            "coo_sparse",
            b"coo_sparse",
            "encoding",
            b"encoding",
            "name",
            b"name",
            "tensor_shape",
            b"tensor_shape",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "composite_tensor",
            b"composite_tensor",
            "coo_sparse",
            b"coo_sparse",
            "dtype",
            b"dtype",
            "encoding",
            b"encoding",
            "name",
            b"name",
            "tensor_shape",
            b"tensor_shape",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["encoding", b"encoding"]
    ) -> typing.Literal["name", "coo_sparse", "composite_tensor"] | None: ...

global___TensorInfo = TensorInfo

@typing.final
class SignatureDef(google.protobuf.message.Message):
    """SignatureDef defines the signature of a computation supported by a TensorFlow
    graph.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class InputsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___TensorInfo: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___TensorInfo | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class OutputsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___TensorInfo: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___TensorInfo | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class DefaultsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> tensorflow.core.framework.tensor_pb2.TensorProto: ...
        def __init__(
            self, *, key: builtins.str | None = ..., value: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    INPUTS_FIELD_NUMBER: builtins.int
    OUTPUTS_FIELD_NUMBER: builtins.int
    METHOD_NAME_FIELD_NUMBER: builtins.int
    DEFAULTS_FIELD_NUMBER: builtins.int
    method_name: builtins.str
    """Deprecated: TensorFlow 2 always sets this to a fixed value;
    open-source TF Serving stopped checking by default since release 2.4.

    In TensorFlow 1, the method_name enabled users to mark a SignatureDef as
    supporting a particular method. Multiple SignatureDefs in a single
    MetaGraphDef could have the same method_name (e.g., to support multi-headed
    computation).
    """
    @property
    def inputs(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___TensorInfo]:
        """Named input parameters."""

    @property
    def outputs(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___TensorInfo]:
        """Named output parameters."""

    @property
    def defaults(
        self,
    ) -> google.protobuf.internal.containers.MessageMap[builtins.str, tensorflow.core.framework.tensor_pb2.TensorProto]:
        """Named input to corresponding default values if any."""

    def __init__(
        self,
        *,
        inputs: collections.abc.Mapping[builtins.str, global___TensorInfo] | None = ...,
        outputs: collections.abc.Mapping[builtins.str, global___TensorInfo] | None = ...,
        method_name: builtins.str | None = ...,
        defaults: collections.abc.Mapping[builtins.str, tensorflow.core.framework.tensor_pb2.TensorProto] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "defaults", b"defaults", "inputs", b"inputs", "method_name", b"method_name", "outputs", b"outputs"
        ],
    ) -> None: ...

global___SignatureDef = SignatureDef

@typing.final
class AssetFileDef(google.protobuf.message.Message):
    """An asset file def for a single file or a set of sharded files with the same
    name.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_INFO_FIELD_NUMBER: builtins.int
    FILENAME_FIELD_NUMBER: builtins.int
    filename: builtins.str
    """The filename within an assets directory. Note: does not include the path
    prefix, i.e. directories. For an asset at /tmp/path/vocab.txt, the filename
    would be "vocab.txt".
    """
    @property
    def tensor_info(self) -> global___TensorInfo:
        """The tensor to bind the asset filename to."""

    def __init__(self, *, tensor_info: global___TensorInfo | None = ..., filename: builtins.str | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["tensor_info", b"tensor_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["filename", b"filename", "tensor_info", b"tensor_info"]) -> None: ...

global___AssetFileDef = AssetFileDef



import re
from collections.abc import Iterable, Iterator
from typing import Any, ClassVar
from typing_extensions import Self

from .connections import Connection

RE_INSERT_VALUES: re.Pattern[str]

class Cursor:
    max_stmt_length: ClassVar[int]
    connection: Connection[Any]
    description: tuple[str, ...]
    rownumber: int
    rowcount: int
    arraysize: int
    messages: Any
    errorhandler: Any
    lastrowid: int
    def __init__(self, connection: Connection[Any]) -> None: ...
    def close(self) -> None: ...
    def setinputsizes(self, *args) -> None: ...
    def setoutputsizes(self, *args) -> None: ...
    def nextset(self) -> bool | None: ...
    def mogrify(self, query: str, args: object = None) -> str: ...
    def execute(self, query: str, args: object = None) -> int: ...
    def executemany(self, query: str, args: Iterable[object]) -> int | None: ...
    def callproc(self, procname: str, args: Iterable[Any] = ()) -> Any: ...
    def scroll(self, value: int, mode: str = "relative") -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...
    # Methods returning result tuples are below.
    def fetchone(self) -> tuple[Any, ...] | None: ...
    def fetchmany(self, size: int | None = None) -> tuple[tuple[Any, ...], ...]: ...
    def fetchall(self) -> tuple[tuple[Any, ...], ...]: ...
    def __iter__(self) -> Iterator[tuple[Any, ...]]: ...
    def __next__(self): ...

class DictCursorMixin:
    dict_type: Any  # TODO: add support if someone needs this
    def fetchone(self) -> dict[str, Any] | None: ...
    def fetchmany(self, size: int | None = ...) -> tuple[dict[str, Any], ...]: ...
    def fetchall(self) -> tuple[dict[str, Any], ...]: ...
    def __iter__(self) -> Iterator[dict[str, Any]]: ...

class SSCursor(Cursor):
    def __del__(self) -> None: ...
    def read_next(self) -> tuple[Any, ...] | None: ...
    def fetchall(self) -> list[tuple[Any, ...]]: ...  # type: ignore[override]
    def fetchall_unbuffered(self) -> Iterator[tuple[Any, ...]]: ...
    def scroll(self, value: int, mode: str = "relative") -> None: ...

class DictCursor(DictCursorMixin, Cursor): ...  # type: ignore[misc]

class SSDictCursor(DictCursorMixin, SSCursor):  # type: ignore[misc]
    def fetchall_unbuffered(self) -> Iterator[dict[str, Any]]: ...  # type: ignore[override]
    def read_next(self) -> dict[str, Any] | None: ...  # type: ignore[override]



from typing import Any

from yaml.error import MarkedYAMLError

class ScannerError(MarkedYAMLError): ...

class SimpleKey:
    token_number: Any
    required: Any
    index: Any
    line: Any
    column: Any
    mark: Any
    def __init__(self, token_number, required, index, line, column, mark) -> None: ...

class Scanner:
    done: Any
    flow_level: Any
    tokens: Any
    tokens_taken: Any
    indent: Any
    indents: Any
    allow_simple_key: Any
    possible_simple_keys: Any
    def __init__(self) -> None: ...
    def check_token(self, *choices): ...
    def peek_token(self): ...
    def get_token(self): ...
    def need_more_tokens(self): ...
    def fetch_more_tokens(self): ...
    def next_possible_simple_key(self): ...
    def stale_possible_simple_keys(self): ...
    def save_possible_simple_key(self): ...
    def remove_possible_simple_key(self): ...
    def unwind_indent(self, column): ...
    def add_indent(self, column): ...
    def fetch_stream_start(self): ...
    def fetch_stream_end(self): ...
    def fetch_directive(self): ...
    def fetch_document_start(self): ...
    def fetch_document_end(self): ...
    def fetch_document_indicator(self, TokenClass): ...
    def fetch_flow_sequence_start(self): ...
    def fetch_flow_mapping_start(self): ...
    def fetch_flow_collection_start(self, TokenClass): ...
    def fetch_flow_sequence_end(self): ...
    def fetch_flow_mapping_end(self): ...
    def fetch_flow_collection_end(self, TokenClass): ...
    def fetch_flow_entry(self): ...
    def fetch_block_entry(self): ...
    def fetch_key(self): ...
    def fetch_value(self): ...
    def fetch_alias(self): ...
    def fetch_anchor(self): ...
    def fetch_tag(self): ...
    def fetch_literal(self): ...
    def fetch_folded(self): ...
    def fetch_block_scalar(self, style): ...
    def fetch_single(self): ...
    def fetch_double(self): ...
    def fetch_flow_scalar(self, style): ...
    def fetch_plain(self): ...
    def check_directive(self): ...
    def check_document_start(self): ...
    def check_document_end(self): ...
    def check_block_entry(self): ...
    def check_key(self): ...
    def check_value(self): ...
    def check_plain(self): ...
    def scan_to_next_token(self): ...
    def scan_directive(self): ...
    def scan_directive_name(self, start_mark): ...
    def scan_yaml_directive_value(self, start_mark): ...
    def scan_yaml_directive_number(self, start_mark): ...
    def scan_tag_directive_value(self, start_mark): ...
    def scan_tag_directive_handle(self, start_mark): ...
    def scan_tag_directive_prefix(self, start_mark): ...
    def scan_directive_ignored_line(self, start_mark): ...
    def scan_anchor(self, TokenClass): ...
    def scan_tag(self): ...
    def scan_block_scalar(self, style): ...
    def scan_block_scalar_indicators(self, start_mark): ...
    def scan_block_scalar_ignored_line(self, start_mark): ...
    def scan_block_scalar_indentation(self): ...
    def scan_block_scalar_breaks(self, indent): ...
    def scan_flow_scalar(self, style): ...
    ESCAPE_REPLACEMENTS: Any
    ESCAPE_CODES: Any
    def scan_flow_scalar_non_spaces(self, double, start_mark): ...
    def scan_flow_scalar_spaces(self, double, start_mark): ...
    def scan_flow_scalar_breaks(self, double, start_mark): ...
    def scan_plain(self): ...
    def scan_plain_spaces(self, indent, start_mark): ...
    def scan_tag_handle(self, name, start_mark): ...
    def scan_tag_uri(self, name, start_mark): ...
    def scan_uri_escapes(self, name, start_mark): ...
    def scan_line_break(self): ...

__all__ = ["Scanner", "ScannerError"]



from typing import TypeVar

from ..base import _ClipMask
from ..geodataframe import GeoDataFrame
from ..geoseries import GeoSeries

_G = TypeVar("_G", GeoDataFrame, GeoSeries)

def clip(gdf: _G, mask: _ClipMask, keep_geom_type: bool = False, sort: bool = False) -> _G: ...



class LZString:
    @staticmethod
    def compress(uncompressed: str | None) -> str: ...
    @staticmethod
    def compressToUTF16(uncompressed: str | None) -> str: ...
    @staticmethod
    def compressToBase64(uncompressed: str | None) -> str: ...
    @staticmethod
    def compressToEncodedURIComponent(uncompressed: str | None) -> str: ...
    @staticmethod
    def decompress(compressed: str | None) -> str | None: ...
    @staticmethod
    def decompressFromUTF16(compressed: str | None) -> str | None: ...
    @staticmethod
    def decompressFromBase64(compressed: str | None) -> str | None: ...
    @staticmethod
    def decompressFromEncodedURIComponent(compressed: str | None) -> str | None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["all_pairs_lowest_common_ancestor", "tree_all_pairs_lowest_common_ancestor", "lowest_common_ancestor"]

@_dispatchable
def all_pairs_lowest_common_ancestor(G: DiGraph[_Node], pairs=None): ...
@_dispatchable
def lowest_common_ancestor(G: DiGraph[_Node], node1, node2, default=None): ...
@_dispatchable
def tree_all_pairs_lowest_common_ancestor(
    G: DiGraph[_Node], root: _Node | None = None, pairs=None
) -> Generator[Incomplete, None, None]: ...



from hvac.api.auth_methods.approle import AppRole as AppRole
from hvac.api.auth_methods.aws import Aws as Aws
from hvac.api.auth_methods.azure import Azure as Azure
from hvac.api.auth_methods.cert import Cert as Cert
from hvac.api.auth_methods.gcp import Gcp as Gcp
from hvac.api.auth_methods.github import Github as Github
from hvac.api.auth_methods.jwt import JWT as JWT
from hvac.api.auth_methods.kubernetes import Kubernetes as Kubernetes
from hvac.api.auth_methods.ldap import Ldap as Ldap
from hvac.api.auth_methods.legacy_mfa import LegacyMfa as LegacyMfa
from hvac.api.auth_methods.oidc import OIDC as OIDC
from hvac.api.auth_methods.okta import Okta as Okta
from hvac.api.auth_methods.radius import Radius as Radius
from hvac.api.auth_methods.token import Token as Token
from hvac.api.auth_methods.userpass import Userpass as Userpass
from hvac.api.vault_api_base import VaultApiBase
from hvac.api.vault_api_category import VaultApiCategory

__all__ = (
    "AuthMethods",
    "AppRole",
    "Azure",
    "Gcp",
    "Github",
    "JWT",
    "Kubernetes",
    "Ldap",
    "Userpass",
    "LegacyMfa",
    "OIDC",
    "Okta",
    "Radius",
    "Token",
    "Aws",
    "Cert",
)

class AuthMethods(VaultApiCategory):
    implemented_classes: list[type[VaultApiBase]]
    unimplemented_classes: list[str]



from _typeshed import Unused
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

from .sources import Source
from .varref import _ValueFormat

@type_check_only
class _StackFrameFormat(_ValueFormat, total=False):
    parameters: bool
    parameterTypes: bool
    parameterNames: bool
    parameterValues: bool
    line: bool
    module: bool
    includeAll: bool

@type_check_only
class _StackFrame(TypedDict):
    id: int
    name: str
    line: int
    column: int
    instructionPointerReference: str
    moduleId: NotRequired[str | None]
    source: NotRequired[Source]

@type_check_only
class _StackTraceResult(TypedDict):
    stackFrames: list[_StackFrame]

def check_stack_frame(
    *,
    # From source:
    #   Note that StackFrameFormat extends ValueFormat, which is why
    #   "hex" appears here.
    hex: bool = False,
    parameters: bool = False,
    parameterTypes: bool = False,
    parameterNames: bool = False,
    parameterValues: bool = False,
    line: bool = False,
    module: bool = False,
    includeAll: bool = False,
    **rest: Unused,
) -> _StackFrameFormat: ...
def stacktrace(
    *, levels: int = 0, startFrame: int = 0, threadId: int, format: _StackFrameFormat | None = None, **extra: Unused
) -> _StackTraceResult: ...



from datetime import date
from typing_extensions import Self, TypeAlias

__tracebackhide__: bool

_Numeric: TypeAlias = date | int | float

class NumericMixin:
    def is_zero(self) -> Self: ...
    def is_not_zero(self) -> Self: ...
    def is_nan(self) -> Self: ...
    def is_not_nan(self) -> Self: ...
    def is_inf(self) -> Self: ...
    def is_not_inf(self) -> Self: ...
    def is_greater_than(self, other: _Numeric) -> Self: ...
    def is_greater_than_or_equal_to(self, other: _Numeric) -> Self: ...
    def is_less_than(self, other: _Numeric) -> Self: ...
    def is_less_than_or_equal_to(self, other: _Numeric) -> Self: ...
    def is_positive(self) -> Self: ...
    def is_negative(self) -> Self: ...
    def is_between(self, low: _Numeric, high: _Numeric) -> Self: ...
    def is_not_between(self, low: _Numeric, high: _Numeric) -> Self: ...
    def is_close_to(self, other: _Numeric, tolerance: _Numeric) -> Self: ...
    def is_not_close_to(self, other: _Numeric, tolerance: _Numeric) -> Self: ...
